@@ -25,6 +25,22 @@
 #define isatty _isatty
 #endif
 
+#ifndef strnicmp
+#define strnicmp _strnicmp
+#endif 
+
+#ifndef stricmp
+#define stricmp _stricmp
+#endif
+
+#ifndef strdup
+#define strdup _strdup
+#endif
+
+#ifndef fileno
+#define fileno _fileno
+#endif
+
 #ifndef mkdir
 #define mkdir _mkdir
 #endif
@@ -33,6 +49,14 @@
 #define access _access
 #endif
 
+#ifndef unlink
+#define unlink _unlink
+#endif
+
+#ifndef putenv
+#define putenv _putenv
+#endif
+
 #ifndef R_OK
 #define R_OK  4		/* test whether readable.  */
 #define W_OK  2		/* test whether writable.  */
@@ -48,7 +72,7 @@
 #define PATH_MAX _MAX_PATH
 
 #ifndef VERSION
-#define VERSION "0.10.0"
+#define VERSION "0.11.1"
 #endif
 
 /* src/common/getpass.c */
