@@ -26,10 +26,10 @@
 #include <assert.h>
 #include <stdlib.h>
 
-static int asn1_decode(struct sc_context *ctx, struct sc_asn1_entry *asn1,
+static int asn1_decode(sc_context_t *ctx, struct sc_asn1_entry *asn1,
 		       const u8 *in, size_t len, const u8 **newp, size_t *len_left,
 		       int choice, int depth);
-static int asn1_encode(struct sc_context *ctx, const struct sc_asn1_entry *asn1,
+static int asn1_encode(sc_context_t *ctx, const struct sc_asn1_entry *asn1,
 		       u8 **ptr, size_t *size, int depth);
 
 static const char *tag2str(unsigned int tag)
@@ -58,28 +58,41 @@ int sc_asn1_read_tag(const u8 ** buf, size_t buflen, unsigned int *cla_out,
 	unsigned int cla, tag, i;
 
 	if (left < 2)
-		goto error;
+		return SC_ERROR_INVALID_ASN1_OBJECT;
 	*buf = NULL;
-	if (*p == 0)
-		return 0;
-	if (*p == 0xFF) /* FIXME */
-		return 0;
-	cla = (*p & ASN1_TAG_CLASS) | (*p & ASN1_TAG_CONSTRUCTED);
-	tag = *p & ASN1_TAG_PRIMITIVE;
-	if (tag == ASN1_TAG_PRIMITIVE) {	/* 0x1F */
-		fprintf(stderr, "Tag number >= 0x1F not supported!\n");
-		goto error;
-	}
+	if (*p == 0xff || *p == 0)
+		/* end of data reached */
+		return SC_SUCCESS;
+	/* parse tag byte(s) */
+	cla = (*p & SC_ASN1_TAG_CLASS) | (*p & SC_ASN1_TAG_CONSTRUCTED);
+	tag = *p & SC_ASN1_TAG_PRIMITIVE;
 	p++;
-	if (--left == 0)
-		goto error;
+	left--;
+	if (tag == SC_ASN1_TAG_PRIMITIVE) {
+		/* high tag number */
+		size_t n = sizeof(int) - 1;
+		/* search the last tag octet */
+		while (left-- != 0 && n != 0) {
+			tag <<= 8;
+			tag |= *p;
+			if ((*p++ & 0x80) == 0)
+				break;
+			n--;
+		}
+		if (left == 0 || n == 0)
+			/* either an invalid tag or it doesn't fit in
+			 * unsigned int */
+			return SC_ERROR_INVALID_ASN1_OBJECT;
+		
+	}
+	if (left == 0)
+		return SC_ERROR_INVALID_ASN1_OBJECT;
+	/* parse length byte(s) */
 	len = *p & 0x7f;
 	if (*p++ & 0x80) {
 		unsigned int a = 0;
-		if (len > 4 || len > left) {
-			fprintf(stderr, "ASN.1 tag too long!\n");
-			goto error;
-		}
+		if (len > 4 || len > left)
+			return SC_ERROR_INVALID_ASN1_OBJECT;
 		left -= len;
 		for (i = 0; i < len; i++) {
 			a <<= 8;
@@ -88,17 +101,13 @@ int sc_asn1_read_tag(const u8 ** buf, size_t buflen, unsigned int *cla_out,
 		}
 		len = a;
 	}
-	if (len > left) {
-		fprintf(stderr, "ASN.1 value too long!\n");
-		goto error;
-	}
+	if (len > left)
+		return SC_ERROR_INVALID_ASN1_OBJECT;
 	*cla_out = cla;
 	*tag_out = tag;
 	*taglen = len;
 	*buf = p;
-	return 1;
-      error:
-	return -1;
+	return SC_SUCCESS;
 }
 
 void sc_format_asn1_entry(struct sc_asn1_entry *entry, void *parm, void *arg,
@@ -121,24 +130,6 @@ void sc_copy_asn1_entry(const struct sc_asn1_entry *src,
 	dest->name = NULL;
 }
 
-size_t _sc_count_bit_string_size(const void * buf, size_t bufsize)
-{
-        const u8 *p = (const u8 *) buf + bufsize - 1;
-        u8 c;
-        size_t skip = 0;
-        int i;
-        
-        while (p >= (const u8 *) buf && *p == 0) {
-                skip += 8;
-                p--;
-        }
-        if (p < (const u8 *) buf)
-                return 0;
-        c = *p;
-        for (i = 0; (c >> (7-i)) == 0; i++);
-        return bufsize * 8 - (skip + i);
-}
-
 static void sc_asn1_print_octet_string(const u8 * buf, size_t buflen)
 {
 	size_t i;
@@ -160,7 +151,7 @@ static void sc_asn1_print_integer(const u8 * buf, size_t buflen)
 #ifndef _WIN32
 	long long a = 0;
 #else
-        __int64 a = 0;
+	__int64 a = 0;
 #endif
 	size_t i;
 
@@ -180,10 +171,9 @@ static void sc_asn1_print_bit_string(const u8 * buf, size_t buflen)
 #ifndef _WIN32
 	long long a = 0;
 #else
-        __int64 a = 0;
+	__int64 a = 0;
 #endif
-	int r;
-	size_t i;
+	int r, i;
 
 	if (buflen > sizeof(a) + 1) {
 		printf("too long");
@@ -238,13 +228,11 @@ static void print_tags_recursive(const u8 * buf0, const u8 * buf,
 		size_t len;
 
 		r = sc_asn1_read_tag(&tagp, bytesleft, &cla, &tag, &len);
-		if (r < 0) {
+		if (r != SC_SUCCESS) {
 			printf("Error in decoding.\n");
 			return;
 		}
 		hlen = tagp - p;
-		if (r == 0)
-			return;
 		if (cla == 0 && tag == 0) {
 			printf("Zero tag, finishing\n");
 			break;
@@ -261,33 +249,33 @@ static void print_tags_recursive(const u8 * buf0, const u8 * buf,
 		}
 		p += hlen + len;
 		bytesleft -= hlen + len;
-		if ((cla & ASN1_TAG_CLASS) == ASN1_TAG_UNIVERSAL)
+		if ((cla & SC_ASN1_TAG_CLASS) == SC_ASN1_TAG_UNIVERSAL)
 			printf("%s", tag2str(tag));
 
-		if (cla & ASN1_TAG_CONSTRUCTED) {
+		if (cla & SC_ASN1_TAG_CONSTRUCTED) {
 			putchar('\n');
 			print_tags_recursive(buf0, tagp, len, depth + 1);
 			continue;
 		}
-		if ((cla & ASN1_TAG_CLASS) == ASN1_TAG_UNIVERSAL) {
+		if ((cla & SC_ASN1_TAG_CLASS) == SC_ASN1_TAG_UNIVERSAL) {
 			printf(" [");
 			switch (tag) {
-			case ASN1_BIT_STRING:
+			case SC_ASN1_TAG_BIT_STRING:
 				sc_asn1_print_bit_string(tagp, len);
 				break;
-			case ASN1_OCTET_STRING:
+			case SC_ASN1_TAG_OCTET_STRING:
 				sc_asn1_print_octet_string(tagp, len);
 				break;
-			case ASN1_OBJECT:
+			case SC_ASN1_TAG_OBJECT:
 				sc_asn1_print_object_id(tagp, len);
 				break;
-			case ASN1_INTEGER:
-			case ASN1_ENUMERATED:
+			case SC_ASN1_TAG_INTEGER:
+			case SC_ASN1_TAG_ENUMERATED:
 				sc_asn1_print_integer(tagp, len);
 				break;
-			case ASN1_T61STRING:
-			case ASN1_PRINTABLESTRING:
-			case ASN1_UTF8STRING:
+			case SC_ASN1_TAG_T61STRING:
+			case SC_ASN1_TAG_PRINTABLESTRING:
+			case SC_ASN1_TAG_UTF8STRING:
 				sc_asn1_print_utf8string(tagp, len);
 				break;
 			}
@@ -304,61 +292,78 @@ void sc_asn1_print_tags(const u8 * buf, size_t buflen)
 	print_tags_recursive(buf, buf, buflen, 0);
 }
 
-const u8 *sc_asn1_find_tag(struct sc_context *ctx, const u8 * buf,
-			   size_t buflen, unsigned int tag_in, size_t *taglen_in)
+const u8 *sc_asn1_find_tag(sc_context_t *ctx, const u8 * buf,
+	size_t buflen, unsigned int tag_in, size_t *taglen_in)
 {
 	size_t left = buflen, taglen;
-	unsigned int cla, tag;
 	const u8 *p = buf;
 
 	*taglen_in = 0;
 	while (left >= 2) {
+		unsigned int cla, tag, mask = 0xff00;
+
 		buf = p;
-		if (sc_asn1_read_tag(&p, left, &cla, &tag, &taglen) != 1)
+		/* read a tag */
+		if (sc_asn1_read_tag(&p, left, &cla, &tag, &taglen) != SC_SUCCESS)
 			return NULL;
-		assert(left >= (p - buf)); /* should not happen */
+		if (left < (size_t)(p - buf)) {
+			sc_error(ctx, "invalid TLV object\n");
+			return NULL;
+		}
 		left -= (p - buf);
+		/* we need to shift the class byte to the leftmost
+		 * byte of the tag */
+		while ((tag & mask) != 0) {
+			cla  <<= 8;
+			mask <<= 8;
+		}
+		/* compare the read tag with the given tag */
 		if ((tag | cla) == tag_in) {
+			/* we have a match => return length and value part */
 			if (taglen > left)
 				return NULL;
 			*taglen_in = taglen;
 			return p;
 		}
-		assert(left >= taglen); /* should not happen */
+		/* otherwise continue reading tags */
+		if (left < taglen) {
+			sc_error(ctx, "invalid TLV object\n");
+			return NULL;
+		}
 		left -= taglen;
 		p += taglen;
 	}
 	return NULL;
 }
 
-const u8 *sc_asn1_skip_tag(struct sc_context *ctx, const u8 ** buf, size_t *buflen,
+const u8 *sc_asn1_skip_tag(sc_context_t *ctx, const u8 ** buf, size_t *buflen,
 			   unsigned int tag_in, size_t *taglen_out)
 {
 	const u8 *p = *buf;
 	size_t len = *buflen, taglen;
 	unsigned int cla, tag;
 
-	if (sc_asn1_read_tag((const u8 **) &p, len, &cla, &tag, &taglen) != 1)
+	if (sc_asn1_read_tag((const u8 **) &p, len, &cla, &tag, &taglen) != SC_SUCCESS)
 		return NULL;
 	switch (cla & 0xC0) {
-	case ASN1_TAG_UNIVERSAL:
+	case SC_ASN1_TAG_UNIVERSAL:
 		if ((tag_in & SC_ASN1_CLASS_MASK) != SC_ASN1_UNI)
 			return NULL;
 		break;
-	case ASN1_TAG_APPLICATION:
+	case SC_ASN1_TAG_APPLICATION:
 		if ((tag_in & SC_ASN1_CLASS_MASK) != SC_ASN1_APP)
 			return NULL;
 		break;
-	case ASN1_TAG_CONTEXT:
+	case SC_ASN1_TAG_CONTEXT:
 		if ((tag_in & SC_ASN1_CLASS_MASK) != SC_ASN1_CTX)
 			return NULL;
 		break;
-	case ASN1_TAG_PRIVATE:
+	case SC_ASN1_TAG_PRIVATE:
 		if ((tag_in & SC_ASN1_CLASS_MASK) != SC_ASN1_PRV)
 			return NULL;
 		break;
 	}
-	if (cla & ASN1_TAG_CONSTRUCTED) {
+	if (cla & SC_ASN1_TAG_CONSTRUCTED) {
 		if ((tag_in & SC_ASN1_CONS) == 0)
 			return NULL;
 	} else
@@ -378,7 +383,7 @@ const u8 *sc_asn1_skip_tag(struct sc_context *ctx, const u8 ** buf, size_t *bufl
 	return p;
 }
 
-const u8 *sc_asn1_verify_tag(struct sc_context *ctx, const u8 * buf, size_t buflen,
+const u8 *sc_asn1_verify_tag(sc_context_t *ctx, const u8 * buf, size_t buflen,
 			     unsigned int tag_in, size_t *taglen_out)
 {
 	return sc_asn1_skip_tag(ctx, &buf, &buflen, tag_in, taglen_out);
@@ -504,7 +509,7 @@ static int encode_bit_field(const u8 *inbuf, size_t inlen,
 {
 	u8		data[sizeof(unsigned int)];
 	unsigned int	field = 0;
-	int		i, bits;
+	size_t		i, bits;
 
 	if (inlen != sizeof(data))
 		return SC_ERROR_BUFFER_TOO_SMALL;
@@ -565,11 +570,11 @@ int sc_asn1_decode_object_id(const u8 * inbuf, size_t inlen,
 {
 	int i, a;
 	const u8 *p = inbuf;
-	int *octet = id->value;
+	int *octet;
 	
-	assert(id != NULL);
-	if (inlen < 1)
-		return SC_ERROR_INVALID_ASN1_OBJECT;
+	if (inlen == 0 || inbuf == NULL || id == NULL)
+		return SC_ERROR_INVALID_ARGUMENTS;
+	octet = id->value;
 	for (i = 0; i < SC_MAX_OBJECT_ID_OCTETS; i++)
 		id->value[i] = -1;
 	a = *p;
@@ -595,12 +600,13 @@ int sc_asn1_decode_object_id(const u8 * inbuf, size_t inlen,
 	return 0;
 }
 
-int sc_asn1_encode_object_id(u8 **buf, size_t *buflen,
+static int sc_asn1_encode_object_id(u8 **buf, size_t *buflen,
 			     const struct sc_object_id *id)
 {
 	u8 temp[SC_MAX_OBJECT_ID_OCTETS*5], *p = temp;
-	size_t count = 0;
-	int *value = (int *) id->value, i = 0;
+	size_t	count = 0;
+	int	i;
+	const int *value = (const int *) id->value;
 
 	for (i = 0; value[i] > 0 && i < SC_MAX_OBJECT_ID_OCTETS; i++) {
 		unsigned int k, shift;
@@ -631,11 +637,13 @@ int sc_asn1_encode_object_id(u8 **buf, size_t *buflen,
 	}
 	*buflen = count = p - temp;
 	*buf = (u8 *) malloc(count);
+	if (!*buf)
+		return SC_ERROR_OUT_OF_MEMORY;
 	memcpy(*buf, temp, count);
 	return 0;
 }
 
-int sc_asn1_decode_utf8string(const u8 *inbuf, size_t inlen,
+static int sc_asn1_decode_utf8string(const u8 *inbuf, size_t inlen,
 			      u8 *out, size_t *outlen)
 {
 	if (inlen+1 > *outlen)
@@ -646,13 +654,13 @@ int sc_asn1_decode_utf8string(const u8 *inbuf, size_t inlen,
 	return 0;
 }
 
-int sc_asn1_put_tag(int tag, const u8 * data, int datalen, u8 * out, int outlen, u8 **ptr)
+int sc_asn1_put_tag(int tag, const u8 * data, size_t datalen, u8 * out, size_t outlen, u8 **ptr)
 {
 	u8 *p = out;
 
 	if (outlen < 2)
 		return SC_ERROR_INVALID_ARGUMENTS;
-	if (datalen < 0 || datalen > 127)
+	if (datalen > 127)
 		return SC_ERROR_INVALID_ARGUMENTS;
 	*p++ = tag & 0xFF;	/* FIXME: Support longer tags */
 	outlen--;
@@ -668,8 +676,8 @@ int sc_asn1_put_tag(int tag, const u8 * data, int datalen, u8 * out, int outlen,
 	return 0;
 }
 
-int asn1_write_element(struct sc_context *ctx, unsigned int tag, const u8 * data,
-		       size_t datalen, u8 ** out, size_t * outlen)
+static int asn1_write_element(sc_context_t *ctx, unsigned int tag,
+	const u8 * data, size_t datalen, u8 ** out, size_t * outlen)
 {
 	u8 t;
 	u8 *buf, *p;
@@ -684,17 +692,17 @@ int asn1_write_element(struct sc_context *ctx, unsigned int tag, const u8 * data
 	case SC_ASN1_UNI:
 		break;
 	case SC_ASN1_APP:
-		t |= ASN1_TAG_APPLICATION;
+		t |= SC_ASN1_TAG_APPLICATION;
 		break;
 	case SC_ASN1_CTX:
-		t |= ASN1_TAG_CONTEXT;
+		t |= SC_ASN1_TAG_CONTEXT;
 		break;
 	case SC_ASN1_PRV:
-		t |= ASN1_TAG_PRIVATE;
+		t |= SC_ASN1_TAG_PRIVATE;
 		break;
 	}
 	if (tag & SC_ASN1_CONS)
-		t |= ASN1_TAG_CONSTRUCTED;
+		t |= SC_ASN1_TAG_CONSTRUCTED;
 	if (datalen > 127) {
 		c = 1;
 		while (datalen >> (c << 3))
@@ -718,14 +726,14 @@ int asn1_write_element(struct sc_context *ctx, unsigned int tag, const u8 * data
 }
 
 static const struct sc_asn1_entry c_asn1_path[4] = {
-	{ "path",   SC_ASN1_OCTET_STRING, ASN1_OCTET_STRING, 0, NULL },
-	{ "index",  SC_ASN1_INTEGER, ASN1_INTEGER, SC_ASN1_OPTIONAL, NULL },
-	{ "length", SC_ASN1_INTEGER, SC_ASN1_CTX | 0, SC_ASN1_OPTIONAL, NULL },
-	{ NULL }
+	{ "path",   SC_ASN1_OCTET_STRING, SC_ASN1_TAG_OCTET_STRING, 0, NULL, NULL },
+	{ "index",  SC_ASN1_INTEGER, SC_ASN1_TAG_INTEGER, SC_ASN1_OPTIONAL, NULL, NULL },
+	{ "length", SC_ASN1_INTEGER, SC_ASN1_CTX | 0, SC_ASN1_OPTIONAL, NULL, NULL },
+	{ NULL, 0, 0, 0, NULL, NULL }
 };
 
-static int asn1_decode_path(struct sc_context *ctx, const u8 *in, size_t len,
-			    struct sc_path *path, int depth)
+static int asn1_decode_path(sc_context_t *ctx, const u8 *in, size_t len,
+			    sc_path_t *path, int depth)
 {
 	int idx, count, r;
 	struct sc_asn1_entry asn1_path[4];
@@ -753,40 +761,41 @@ static int asn1_decode_path(struct sc_context *ctx, const u8 *in, size_t len,
 	return 0;
 }
 
-static int asn1_encode_path(struct sc_context *ctx, const struct sc_path *path,
+static int asn1_encode_path(sc_context_t *ctx, const sc_path_t *path,
 			    u8 **buf, size_t *bufsize, int depth)
 {
 	int r;
  	struct sc_asn1_entry asn1_path[4];
+	sc_path_t tpath = *path;
 
 	sc_copy_asn1_entry(c_asn1_path, asn1_path);
-	sc_format_asn1_entry(asn1_path + 0, (void *) &path->value, (void *) &path->len, 1);
+	sc_format_asn1_entry(asn1_path + 0, (void *) &tpath.value, (void *) &tpath.len, 1);
 	if (path->count > 0) {
-		sc_format_asn1_entry(asn1_path + 1, (void *) &path->index, NULL, 1);
-		sc_format_asn1_entry(asn1_path + 2, (void *) &path->count, NULL, 1);
+		sc_format_asn1_entry(asn1_path + 1, (void *) &tpath.index, NULL, 1);
+		sc_format_asn1_entry(asn1_path + 2, (void *) &tpath.count, NULL, 1);
 	}
 	r = asn1_encode(ctx, asn1_path, buf, bufsize, depth + 1);
 	return r;	
 }
 
 static const struct sc_asn1_entry c_asn1_com_obj_attr[6] = {
-	{ "label", SC_ASN1_UTF8STRING, ASN1_UTF8STRING, SC_ASN1_OPTIONAL, NULL },
-	{ "flags", SC_ASN1_BIT_FIELD, ASN1_BIT_STRING, SC_ASN1_OPTIONAL, NULL },
-	{ "authId", SC_ASN1_PKCS15_ID, ASN1_OCTET_STRING, SC_ASN1_OPTIONAL, NULL },
-	{ "userConsent", SC_ASN1_INTEGER, ASN1_INTEGER, SC_ASN1_OPTIONAL, NULL },
-	{ "accessControlRules", SC_ASN1_STRUCT, ASN1_SEQUENCE | SC_ASN1_CONS, SC_ASN1_OPTIONAL, NULL },
-	{ NULL }
+	{ "label", SC_ASN1_UTF8STRING, SC_ASN1_TAG_UTF8STRING, SC_ASN1_OPTIONAL, NULL, NULL },
+	{ "flags", SC_ASN1_BIT_FIELD, SC_ASN1_TAG_BIT_STRING, SC_ASN1_OPTIONAL, NULL, NULL },
+	{ "authId", SC_ASN1_PKCS15_ID, SC_ASN1_TAG_OCTET_STRING, SC_ASN1_OPTIONAL, NULL, NULL },
+	{ "userConsent", SC_ASN1_INTEGER, SC_ASN1_TAG_INTEGER, SC_ASN1_OPTIONAL, NULL, NULL },
+	{ "accessControlRules", SC_ASN1_STRUCT, SC_ASN1_TAG_SEQUENCE | SC_ASN1_CONS, SC_ASN1_OPTIONAL, NULL, NULL },
+	{ NULL, 0, 0, 0, NULL, NULL }
 };
 
 static const struct sc_asn1_entry c_asn1_p15_obj[5] = {
-	{ "commonObjectAttributes", SC_ASN1_STRUCT, ASN1_SEQUENCE | SC_ASN1_CONS, 0, NULL },
-	{ "classAttributes", SC_ASN1_STRUCT, ASN1_SEQUENCE | SC_ASN1_CONS, 0, NULL },
-	{ "subClassAttributes", SC_ASN1_STRUCT, SC_ASN1_CTX | 0 | SC_ASN1_CONS, SC_ASN1_OPTIONAL, NULL },
-	{ "typeAttributes", SC_ASN1_STRUCT, SC_ASN1_CTX | 1 | SC_ASN1_CONS, 0, NULL },
-	{ NULL }
+	{ "commonObjectAttributes", SC_ASN1_STRUCT, SC_ASN1_TAG_SEQUENCE | SC_ASN1_CONS, 0, NULL, NULL },
+	{ "classAttributes", SC_ASN1_STRUCT, SC_ASN1_TAG_SEQUENCE | SC_ASN1_CONS, 0, NULL, NULL },
+	{ "subClassAttributes", SC_ASN1_STRUCT, SC_ASN1_CTX | 0 | SC_ASN1_CONS, SC_ASN1_OPTIONAL, NULL, NULL },
+	{ "typeAttributes", SC_ASN1_STRUCT, SC_ASN1_CTX | 1 | SC_ASN1_CONS, 0, NULL, NULL },
+	{ NULL, 0, 0, 0, NULL, NULL }
 };
 
-static int asn1_decode_p15_object(struct sc_context *ctx, const u8 *in,
+static int asn1_decode_p15_object(sc_context_t *ctx, const u8 *in,
 				  size_t len, struct sc_asn1_pkcs15_object *obj,
 				  int depth)
 {
@@ -813,27 +822,27 @@ static int asn1_decode_p15_object(struct sc_context *ctx, const u8 *in,
 	return r;
 }
 
-static int asn1_encode_p15_object(struct sc_context *ctx, const struct sc_asn1_pkcs15_object *obj,
+static int asn1_encode_p15_object(sc_context_t *ctx, const struct sc_asn1_pkcs15_object *obj,
 				  u8 **buf, size_t *bufsize, int depth)
 {
 	int r;
-	const struct sc_pkcs15_object *p15_obj = obj->p15_obj;
-	struct sc_asn1_entry asn1_c_attr[6], asn1_p15_obj[5];
-	size_t label_len = strlen(p15_obj->label);
+	struct sc_pkcs15_object p15_obj = *obj->p15_obj;
+	struct sc_asn1_entry    asn1_c_attr[6], asn1_p15_obj[5];
+	size_t label_len = strlen(p15_obj.label);
 	size_t flags_len;
 
 	sc_copy_asn1_entry(c_asn1_com_obj_attr, asn1_c_attr);
 	sc_copy_asn1_entry(c_asn1_p15_obj, asn1_p15_obj);
 	if (label_len != 0)
-		sc_format_asn1_entry(asn1_c_attr + 0, (void *) p15_obj->label, &label_len, 1);
-	if (p15_obj->flags) {
-		flags_len = sizeof(p15_obj->flags);
-		sc_format_asn1_entry(asn1_c_attr + 1, (void *) &p15_obj->flags, &flags_len, 1);
+		sc_format_asn1_entry(asn1_c_attr + 0, (void *) p15_obj.label, &label_len, 1);
+	if (p15_obj.flags) {
+		flags_len = sizeof(p15_obj.flags);
+		sc_format_asn1_entry(asn1_c_attr + 1, (void *) &p15_obj.flags, &flags_len, 1);
 	}
-	if (p15_obj->auth_id.len)
-		sc_format_asn1_entry(asn1_c_attr + 2, (void *) &p15_obj->auth_id, NULL, 1);
-	if (p15_obj->user_consent)
-		sc_format_asn1_entry(asn1_c_attr + 3, (void *) &p15_obj->user_consent, NULL, 1);
+	if (p15_obj.auth_id.len)
+		sc_format_asn1_entry(asn1_c_attr + 2, (void *) &p15_obj.auth_id, NULL, 1);
+	if (p15_obj.user_consent)
+		sc_format_asn1_entry(asn1_c_attr + 3, (void *) &p15_obj.user_consent, NULL, 1);
 	/* FIXME: decode accessControlRules */
 	sc_format_asn1_entry(asn1_p15_obj + 0, asn1_c_attr, NULL, 1);
 	sc_format_asn1_entry(asn1_p15_obj + 1, obj->asn1_class_attr, NULL, 1);
@@ -845,16 +854,17 @@ static int asn1_encode_p15_object(struct sc_context *ctx, const struct sc_asn1_p
 	return r;
 }
 
-static int asn1_decode_entry(struct sc_context *ctx, struct sc_asn1_entry *entry,
+static int asn1_decode_entry(sc_context_t *ctx,struct sc_asn1_entry *entry,
 			     const u8 *obj, size_t objlen, int depth)
 {
 	void *parm = entry->parm;
-	int (*callback_func)(struct sc_context *ctx, void *arg, const u8 *obj,
-			     size_t objlen, int depth) =
-		(int (*)(struct sc_context *, void *, const u8 *, size_t, int)) parm;
+	int (*callback_func)(sc_context_t *nctx, void *arg, const u8 *nobj,
+			     size_t nobjlen, int ndepth); 
 	size_t *len = (size_t *) entry->arg;
 	int r = 0;
 
+	*(void **)(&callback_func) = parm;
+
 	if (ctx->debug >= 3)
 		sc_debug(ctx, "%*.*sdecoding '%s'\n", depth, depth, "", entry->name);
 
@@ -912,7 +922,7 @@ static int asn1_decode_entry(struct sc_context *ctx, struct sc_asn1_entry *entry
 		break;
 	case SC_ASN1_OCTET_STRING:
 		if (parm != NULL) {
-			int c;
+			size_t c;
 			assert(len != NULL);
 
 			/* Strip off padding zero */
@@ -940,10 +950,8 @@ static int asn1_decode_entry(struct sc_context *ctx, struct sc_asn1_entry *entry
 		}
 		break;
 	case SC_ASN1_GENERALIZEDTIME:
-                /* FIXME: we should parse the string and convert it
-                   into a standard ISO time string. */
 		if (parm != NULL) {
-			int c;
+			size_t c;
 			assert(len != NULL);
 			if (entry->flags & SC_ASN1_ALLOC) {
 				u8 **buf = (u8 **) parm;
@@ -965,6 +973,7 @@ static int asn1_decode_entry(struct sc_context *ctx, struct sc_asn1_entry *entry
 		if (parm != NULL)
 			r = sc_asn1_decode_object_id(obj, objlen, (struct sc_object_id *) parm);
 		break;
+	case SC_ASN1_PRINTABLESTRING:
 	case SC_ASN1_UTF8STRING:
 		if (parm != NULL) {
 			assert(len != NULL);
@@ -983,12 +992,12 @@ static int asn1_decode_entry(struct sc_context *ctx, struct sc_asn1_entry *entry
 		break;
 	case SC_ASN1_PATH:
 		if (entry->parm != NULL)
-			r = asn1_decode_path(ctx, obj, objlen, (struct sc_path *) parm, depth);
+			r = asn1_decode_path(ctx, obj, objlen, (sc_path_t *) parm, depth);
 		break;
 	case SC_ASN1_PKCS15_ID:
 		if (entry->parm != NULL) {
 			struct sc_pkcs15_id *id = (struct sc_pkcs15_id *) parm;
-			int c = objlen > sizeof(id->value) ? sizeof(id->value) : objlen;
+			size_t c = objlen > sizeof(id->value) ? sizeof(id->value) : objlen;
 			
 			memcpy(id->value, obj, c);
 			id->len = c;
@@ -1001,14 +1010,14 @@ static int asn1_decode_entry(struct sc_context *ctx, struct sc_asn1_entry *entry
 	case SC_ASN1_ALGORITHM_ID:
 		if (entry->parm != NULL)
 			r = sc_asn1_decode_algorithm_id(ctx, obj, objlen, (struct sc_algorithm_id *) parm, depth);
-                break;
+		break;
 	case SC_ASN1_CALLBACK:
 		if (entry->parm != NULL)
 			r = callback_func(ctx, entry->arg, obj, objlen, depth);
 		break;
 	default:
 		sc_error(ctx, "invalid ASN.1 type: %d\n", entry->type);
-		assert(0);
+		return SC_ERROR_INVALID_ASN1_OBJECT;
 	}
 	if (r) {
 		sc_error(ctx, "decoding of ASN.1 object '%s' failed: %s\n", entry->name,
@@ -1019,7 +1028,7 @@ static int asn1_decode_entry(struct sc_context *ctx, struct sc_asn1_entry *entry
 	return 0;
 }
 
-static int asn1_decode(struct sc_context *ctx, struct sc_asn1_entry *asn1,
+static int asn1_decode(sc_context_t *ctx, struct sc_asn1_entry *asn1,
 		       const u8 *in, size_t len, const u8 **newp, size_t *len_left,
 		       int choice, int depth)
 {
@@ -1046,7 +1055,7 @@ static int asn1_decode(struct sc_context *ctx, struct sc_asn1_entry *asn1,
 			      asn1->name);
 		return SC_ERROR_ASN1_OBJECT_NOT_FOUND;
 	}
-	if (p[0] == 0 || p[0] == 0xFF)
+	if (p[0] == 0 || p[0] == 0xFF || len == 0)
 		return SC_ERROR_ASN1_END_OF_CONTENTS;
 
 	for (idx = 0; asn1[idx].name != NULL; idx++) {
@@ -1111,30 +1120,31 @@ decode_ok:
 	SC_FUNC_RETURN(ctx, 3, 0);
 }
 
-int sc_asn1_decode(struct sc_context *ctx, struct sc_asn1_entry *asn1,
+int sc_asn1_decode(sc_context_t *ctx, struct sc_asn1_entry *asn1,
 		   const u8 *in, size_t len, const u8 **newp, size_t *len_left)
 {
 	return asn1_decode(ctx, asn1, in, len, newp, len_left, 0, 0);
 }
 
-int sc_asn1_decode_choice(struct sc_context *ctx, struct sc_asn1_entry *asn1,
+int sc_asn1_decode_choice(sc_context_t *ctx, struct sc_asn1_entry *asn1,
 			  const u8 *in, size_t len, const u8 **newp, size_t *len_left)
 {
 	return asn1_decode(ctx, asn1, in, len, newp, len_left, 1, 0);
 }
 
-static int asn1_encode_entry(struct sc_context *ctx, const struct sc_asn1_entry *entry,
+static int asn1_encode_entry(sc_context_t *ctx, const struct sc_asn1_entry *entry,
 			     u8 **obj, size_t *objlen, int depth)
 {
 	void *parm = entry->parm;
-	int (*callback_func)(struct sc_context *ctx, void *arg, u8 **obj,
-			     size_t *objlen, int depth) =
-		(int (*)(struct sc_context *, void *, u8 **, size_t *, int)) parm;
+	int (*callback_func)(sc_context_t *nctx, void *arg, u8 **nobj,
+			     size_t *nobjlen, int ndepth);
 	const size_t *len = (const size_t *) entry->arg;
 	int r = 0;
 	u8 * buf = NULL;
 	size_t buflen = 0;
 
+	*(void **)(&callback_func) = parm;
+
 	if (ctx->debug >= 3)
 		sc_debug(ctx, "%*.*sencoding '%s'%s\n",
 			       	depth, depth, "",
@@ -1172,7 +1182,11 @@ static int asn1_encode_entry(struct sc_context *ctx, const struct sc_asn1_entry
 		return asn1_encode_entry(ctx, choice, obj, objlen, depth + 1);
 	}
 
-	assert(entry->type == SC_ASN1_NULL || parm != NULL);
+	if (entry->type != SC_ASN1_NULL && parm == NULL) {
+		sc_error(ctx, "unexpected parm == NULL\n");
+		return SC_ERROR_INVALID_ASN1_OBJECT;
+	}
+
 	switch (entry->type) {
 	case SC_ASN1_STRUCT:
 		r = asn1_encode(ctx, (const struct sc_asn1_entry *) parm, &buf,
@@ -1207,6 +1221,7 @@ static int asn1_encode_entry(struct sc_context *ctx, const struct sc_asn1_entry
 		assert(len != NULL);
 		r = encode_bit_field((const u8 *) parm, *len, &buf, &buflen);
 		break;
+	case SC_ASN1_PRINTABLESTRING:
 	case SC_ASN1_OCTET_STRING:
 	case SC_ASN1_UTF8STRING:
 		assert(len != NULL);
@@ -1225,15 +1240,24 @@ static int asn1_encode_entry(struct sc_context *ctx, const struct sc_asn1_entry
 		memcpy(buf + buflen, parm, *len);
 		buflen += *len;
 		break;
+	case SC_ASN1_GENERALIZEDTIME:
+		assert(len != NULL);
+		buf = (u8 *) malloc(*len);
+		if (buf == NULL) {
+			r = SC_ERROR_OUT_OF_MEMORY;
+			break;
+		}
+		memcpy(buf, parm, *len);
+		buflen = *len;
+		break;
 	case SC_ASN1_OBJECT:
-		if (parm != NULL)
-			r = sc_asn1_encode_object_id(&buf, &buflen, (struct sc_object_id *) parm);
+		r = sc_asn1_encode_object_id(&buf, &buflen, (struct sc_object_id *) parm);
 		break;
 	case SC_ASN1_PATH:
-		r = asn1_encode_path(ctx, (const struct sc_path *) parm, &buf, &buflen, depth);
+		r = asn1_encode_path(ctx, (const sc_path_t *) parm, &buf, &buflen, depth);
 		break;
 	case SC_ASN1_PKCS15_ID:
-		if (entry->parm != NULL) {
+		{
 			const struct sc_pkcs15_id *id = (const struct sc_pkcs15_id *) parm;
 
 			buf = (u8 *) malloc(id->len);
@@ -1256,7 +1280,7 @@ static int asn1_encode_entry(struct sc_context *ctx, const struct sc_asn1_entry
 		break;
 	default:
 		sc_error(ctx, "invalid ASN.1 type: %d\n", entry->type);
-		assert(0);
+		return SC_ERROR_INVALID_ASN1_OBJECT;
 	}
 	if (r) {
 		sc_error(ctx, "encoding of ASN.1 object '%s' failed: %s\n", entry->name,
@@ -1305,7 +1329,7 @@ no_object:
 	return r;
 }
 
-static int asn1_encode(struct sc_context *ctx, const struct sc_asn1_entry *asn1,
+static int asn1_encode(sc_context_t *ctx, const struct sc_asn1_entry *asn1,
 		      u8 **ptr, size_t *size, int depth)
 {
 	int r, idx = 0;
@@ -1336,6 +1360,7 @@ static int asn1_encode(struct sc_context *ctx, const struct sc_asn1_entry *asn1,
 		buf = tmp;
 		memcpy(buf + total, obj, objsize);
 		free(obj);
+		obj = NULL;
 		total += objsize;
 	}
 	*ptr = buf;
@@ -1343,20 +1368,20 @@ static int asn1_encode(struct sc_context *ctx, const struct sc_asn1_entry *asn1,
 	return 0;
 }
 
-int sc_asn1_encode(struct sc_context *ctx, const struct sc_asn1_entry *asn1,
+int sc_asn1_encode(sc_context_t *ctx, const struct sc_asn1_entry *asn1,
 		   u8 **ptr, size_t *size)
 {
 	return asn1_encode(ctx, asn1, ptr, size, 0);
 }
 
-int _sc_asn1_encode(struct sc_context *ctx, const struct sc_asn1_entry *asn1,
+int _sc_asn1_encode(sc_context_t *ctx, const struct sc_asn1_entry *asn1,
 		    u8 **ptr, size_t *size, int depth)
 {
 	return asn1_encode(ctx, asn1, ptr, size, depth);
 }
 
 int
-_sc_asn1_decode(struct sc_context *ctx, struct sc_asn1_entry *asn1,
+_sc_asn1_decode(sc_context_t *ctx, struct sc_asn1_entry *asn1,
 		       const u8 *in, size_t len, const u8 **newp, size_t *left,
 		       int choice, int depth)
 {
