@@ -26,54 +26,40 @@
 
 static struct sc_atr_table miocos_atrs[] = {
 	/* Test card with 32 kB memory */
-	{ (const u8 *) "\x3B\x9D\x94\x40\x23\x00\x68\x10\x11\x4D\x69\x6F\x43\x4F\x53\x00\x90\x00", 18 },
+	{ "3B:9D:94:40:23:00:68:10:11:4D:69:6F:43:4F:53:00:90:00", NULL, NULL, SC_CARD_TYPE_MIOCOS_GENERIC, 0, NULL },
 	/* Test card with 64 kB memory */
-	{ (const u8 *) "\x3B\x9D\x94\x40\x23\x00\x68\x20\x01\x4D\x69\x6F\x43\x4F\x53\x00\x90\x00", 18 },
-	{ NULL }
+	{ "3B:9D:94:40:23:00:68:20:01:4D:69:6F:43:4F:53:00:90:00", NULL, NULL, SC_CARD_TYPE_MIOCOS_GENERIC, 0, NULL },
+	{ NULL, NULL, NULL, 0, 0, NULL }
 };
 
-struct miocos_priv_data {
-	int type;
-};
-
-#define DRVDATA(card)        ((struct miocos_priv_data *) ((card)->drv_data))
-
 static struct sc_card_operations miocos_ops;
 static struct sc_card_driver miocos_drv = {
-	"MioCOS 1.1 cards",
+	"MioCOS 1.1",
 	"miocos",
-	&miocos_ops
+	&miocos_ops,
+	NULL, 0, NULL
 };
 
-static int miocos_finish(struct sc_card *card)
+static int miocos_finish(sc_card_t *card)
 {
 	return 0;
 }
 
-static int miocos_match_card(struct sc_card *card)
+static int miocos_match_card(sc_card_t *card)
 {
 	int i;
 
-	i = _sc_match_atr(card, miocos_atrs, NULL);
+	i = _sc_match_atr(card, miocos_atrs, &card->type);
 	if (i < 0)
 		return 0;
 	return 1;
 }
 
-static int miocos_init(struct sc_card *card)
+static int miocos_init(sc_card_t *card)
 {
-	int i, id;
-	struct miocos_priv_data *priv = NULL;
-
-	i = _sc_match_atr(card, miocos_atrs, &id);
-	if (i < 0)
-		return 0;
-	priv = (struct miocos_priv_data *) malloc(sizeof(struct miocos_priv_data));
-	if (priv == NULL)
-		return SC_ERROR_OUT_OF_MEMORY;
 	card->name = "MioCOS";
-	card->drv_data = priv;
 	card->cla = 0x00;
+
 	if (1) {
 		unsigned long flags;
 		
@@ -95,7 +81,7 @@ static int miocos_init(struct sc_card *card)
 
 static const struct sc_card_operations *iso_ops = NULL;
 
-static int acl_to_byte(const struct sc_acl_entry *e)
+static int acl_to_byte(const sc_acl_entry_t *e)
 {
 	switch (e->method) {
 	case SC_AC_NONE:
@@ -114,7 +100,7 @@ static int acl_to_byte(const struct sc_acl_entry *e)
 	return 0x00;
 }
 
-static int encode_file_structure(struct sc_card *card, const struct sc_file *file,
+static int encode_file_structure(sc_card_t *card, const sc_file_t *file,
 				 u8 *buf, size_t *buflen)
 {
 	u8 *p = buf;
@@ -217,9 +203,9 @@ static int encode_file_structure(struct sc_card *card, const struct sc_file *fil
         return 0;
 }
 
-static int miocos_create_file(struct sc_card *card, struct sc_file *file)
+static int miocos_create_file(sc_card_t *card, sc_file_t *file)
 {
-	struct sc_apdu apdu;
+	sc_apdu_t apdu;
 	u8 sbuf[32];
         size_t buflen;
 	int r;
@@ -242,12 +228,12 @@ static int miocos_create_file(struct sc_card *card, struct sc_file *file)
 	return 0;
 }
 
-static int miocos_set_security_env(struct sc_card *card,
-				  const struct sc_security_env *env,
+static int miocos_set_security_env(sc_card_t *card,
+				  const sc_security_env_t *env,
 				  int se_num)
 {
 	if (env->flags & SC_SEC_ENV_ALG_PRESENT) {
-		struct sc_security_env tmp;
+		sc_security_env_t tmp;
 
 		tmp = *env;
 		tmp.flags &= ~SC_SEC_ENV_ALG_PRESENT;
@@ -269,7 +255,7 @@ static int miocos_set_security_env(struct sc_card *card,
 	return iso_ops->set_security_env(card, env, se_num);
 }
 
-static void add_acl_entry(struct sc_file *file, int op, u8 byte)
+static void add_acl_entry(sc_file_t *file, int op, u8 byte)
 {
 	unsigned int method, key_ref = SC_AC_KEY_REF_NONE;
 
@@ -288,7 +274,7 @@ static void add_acl_entry(struct sc_file *file, int op, u8 byte)
 	sc_file_add_acl_entry(file, op, method, key_ref);
 }
 
-static void parse_sec_attr(struct sc_file *file, const u8 *buf, size_t len)
+static void parse_sec_attr(sc_file_t *file, const u8 *buf, size_t len)
 {
 	int i;
 	const int df_ops[8] = {
@@ -332,13 +318,14 @@ static void parse_sec_attr(struct sc_file *file, const u8 *buf, size_t len)
 	}
 }
 
-static int miocos_get_acl(struct sc_card *card, struct sc_file *file)
+static int miocos_get_acl(sc_card_t *card, sc_file_t *file)
 {
-	struct sc_apdu apdu;
+	sc_apdu_t apdu;
 	u8 rbuf[256];
 	const u8 *seq = rbuf;
 	size_t left;
-	int acl_types[16], r, i;
+	int acl_types[16], r;
+	unsigned int i;
 	
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_2_SHORT, 0xCA, 0x01, 0x01);
 	apdu.resp = rbuf;
@@ -366,9 +353,9 @@ static int miocos_get_acl(struct sc_card *card, struct sc_file *file)
 		if (tag == NULL || taglen == 0)
 			continue;
 		for (j = 0; j < SC_MAX_AC_OPS; j++) {
-			struct sc_acl_entry *e;
+			sc_acl_entry_t *e;
 			
-			e = (struct sc_acl_entry *) sc_file_get_acl_entry(file, j);
+			e = (sc_acl_entry_t *) sc_file_get_acl_entry(file, j);
 			if (e == NULL)
 				continue;
 			if (e->method != SC_AC_CHV)
@@ -391,9 +378,9 @@ static int miocos_get_acl(struct sc_card *card, struct sc_file *file)
 	return 0;
 }
 
-static int miocos_select_file(struct sc_card *card,
-			       const struct sc_path *in_path,
-			       struct sc_file **file)
+static int miocos_select_file(sc_card_t *card,
+			       const sc_path_t *in_path,
+			       sc_file_t **file)
 {
 	int r;
 
@@ -408,9 +395,9 @@ static int miocos_select_file(struct sc_card *card,
 	return 0;
 }
 
-static int miocos_list_files(struct sc_card *card, u8 *buf, size_t buflen)
+static int miocos_list_files(sc_card_t *card, u8 *buf, size_t buflen)
 {
-	struct sc_apdu apdu;
+	sc_apdu_t apdu;
 	int r;
 
 	sc_format_apdu(card, &apdu, SC_APDU_CASE_2_SHORT, 0xCA, 0x01, 0);
@@ -424,10 +411,10 @@ static int miocos_list_files(struct sc_card *card, u8 *buf, size_t buflen)
 	return apdu.resplen;
 }
 
-static int miocos_delete_file(struct sc_card *card, const struct sc_path *path)
+static int miocos_delete_file(sc_card_t *card, const sc_path_t *path)
 {
 	int r;
-	struct sc_apdu apdu;
+	sc_apdu_t apdu;
 
 	SC_FUNC_CALLED(card->ctx, 1);
 	if (path->type != SC_PATH_TYPE_FILE_ID && path->len != 2) {
@@ -448,7 +435,7 @@ static int miocos_delete_file(struct sc_card *card, const struct sc_path *path)
 static int miocos_create_ac(sc_card_t *card,
 			    struct sc_cardctl_miocos_ac_info *ac)
 {
-	struct sc_apdu apdu;
+	sc_apdu_t apdu;
 	u8 sbuf[20];
 	int miocos_type, r;
 	size_t sendsize;
@@ -482,7 +469,7 @@ static int miocos_create_ac(sc_card_t *card,
 	return sc_check_sw(card, apdu.sw1, apdu.sw2);
 }
 
-static int miocos_card_ctl(struct sc_card *card, unsigned long cmd,
+static int miocos_card_ctl(sc_card_t *card, unsigned long cmd,
 			   void *arg)
 {
 	switch (cmd) {
