@@ -170,6 +170,7 @@ auth_select_aid(sc_card_t *card)
 	apdu.le = 0x2D;
 	apdu.resplen = 0x30;
 	apdu.resp = apdu_resp;
+	/* FIXME: check return value of this sc_transmit_apdu call? */
 	sc_transmit_apdu(card, &apdu);
 	if (apdu.sw1==0x90)  {
 		card->serialnr.len = 4;
@@ -737,10 +738,22 @@ auth_select_file(sc_card_t *card, const sc_path_t *path,
 	int locked = 0, magic_done;
 	u8 p1 = 0;
 
-	sc_debug(card->ctx, "path; type=%d, path=%s\n",
-			path->type, sc_print_path(path));
-	sc_debug(card->ctx, "cache; type=%d, path=%s\n",
-			card->cache.current_path.type, sc_print_path(&card->cache.current_path));
+	if (card->ctx->debug >= 1) {
+		char pbuf[SC_MAX_PATH_STRING_SIZE];
+
+		rv = sc_path_print(pbuf, sizeof(pbuf), path);
+		if (rv != SC_SUCCESS)
+			pbuf[0] = '\0';
+
+		sc_debug(card->ctx, "path; type=%d, path=%s\n", path->type, pbuf);
+
+		rv = sc_path_print(pbuf, sizeof(pbuf), &card->cache.current_path);
+		if (rv != SC_SUCCESS)
+			pbuf[0] = '\0';
+
+		sc_debug(card->ctx, "cache; type=%d, path=%s\n",
+			card->cache.current_path.type, pbuf);
+	}
 	
 	switch (path->type) {
 	case SC_PATH_TYPE_PATH:
@@ -832,7 +845,16 @@ auth_delete_file(sc_card_t *card, const sc_path_t *path)
 	u8 sbuf[2];
 	sc_apdu_t apdu;
 
-	sc_debug(card->ctx, "path; type=%d, path=%s\n", path->type, sc_print_path(path));
+	if (card->ctx->debug >= 1) {
+		char pbuf[SC_MAX_PATH_STRING_SIZE];
+
+		rv = sc_path_print(pbuf, sizeof(pbuf), path);
+		if (rv != SC_SUCCESS)
+			pbuf[0] = '\0';
+
+		sc_debug(card->ctx, "path; type=%d, path=%s\n", path->type, pbuf);
+	}
+
 	SC_FUNC_CALLED(card->ctx, 1);
 	if (path->len < 2)   {
 		sc_error(card->ctx, "Invalid path length\n");
@@ -1135,16 +1157,28 @@ auth_create_file(sc_card_t *card, sc_file_t *file)
 	int rv, rec_nr;
 	sc_apdu_t apdu;
 	sc_path_t path;
+	char pbuf[SC_MAX_PATH_STRING_SIZE];
 	struct auth_private_data *prv = (struct auth_private_data *) card->drv_data;
 
-	sc_debug(card->ctx, " create path=%s\n", sc_print_path(&file->path));
-	sc_debug(card->ctx,"id %04X; size %i; type %i; ef %i\n",
+	if (card->ctx->debug >= 1) {
+		rv = sc_path_print(pbuf, sizeof(pbuf), &file->path);
+		if (rv != SC_SUCCESS)
+			pbuf[0] = '\0';
+
+		sc_debug(card->ctx, " create path=%s\n", pbuf);
+		sc_debug(card->ctx,"id %04X; size %i; type %i; ef %i\n",
 			file->id, file->size, file->type, file->ef_structure);
+	}
+
 	if (file->id==0x0000 || file->id==0xFFFF || file->id==0x3FFF)  
 		return SC_ERROR_INVALID_ARGUMENTS;
 
-	sc_debug(card->ctx, " cache path=%s\n", 
-			sc_print_path(&card->cache.current_path));
+	if (card->ctx->debug >= 1) {
+		rv = sc_path_print(pbuf, sizeof(pbuf), &card->cache.current_path);
+		if (rv != SC_SUCCESS)
+			pbuf[0] = '\0';
+		sc_debug(card->ctx, " cache path=%s\n", pbuf);
+	}
 
 	if (file->path.len)   {
 		memcpy(&path, &file->path, sizeof(path));
@@ -1235,8 +1269,10 @@ auth_set_security_env(sc_card_t *card,
 		sc_debug(card->ctx, "algo SC_ALGORITHM_xDES: ref %X, flags %X\n", 
 				env->algorithm_ref, env->flags);
 		if (key_file->ef_structure != SC_CARDCTL_OBERTHUR_KEY_DES || 
-				key_file->type != SC_FILE_TYPE_INTERNAL_EF)
+				key_file->type != SC_FILE_TYPE_INTERNAL_EF) {
+			sc_file_free(key_file);
 			return SC_ERROR_INVALID_ARGUMENTS;
+		}
 
 		des_buf_len = 3;
 		if (env->flags & SC_SEC_ENV_FILE_REF_PRESENT)   {
@@ -1252,6 +1288,7 @@ auth_set_security_env(sc_card_t *card,
 			apdu.datalen = des_buf_len;
 		}
 		else {
+			sc_file_free(key_file);
 			sc_error(card->ctx, "Invalid crypto operation: %X\n", env->operation);
 			return SC_ERROR_NOT_SUPPORTED;
 		}
@@ -1260,18 +1297,22 @@ auth_set_security_env(sc_card_t *card,
 	case SC_ALGORITHM_RSA:
 		sc_debug(card->ctx, "algo SC_ALGORITHM_RSA\n");
 		if (env->algorithm_flags & SC_ALGORITHM_RSA_HASHES) {
+			sc_file_free(key_file);
 			sc_error(card->ctx, "Not support for hashes.\n");
 			return SC_ERROR_NOT_SUPPORTED;
 		}
 		
 		if (pads & (~supported_pads))   {
+			sc_file_free(key_file);
 			sc_error(card->ctx, "No support for this PAD: %X\n",pads);
 			return SC_ERROR_NOT_SUPPORTED;
 		}
 	
 		if (key_file->type != SC_FILE_TYPE_INTERNAL_EF ||  
-				key_file->ef_structure != SC_CARDCTL_OBERTHUR_KEY_RSA_CRT)
+				key_file->ef_structure != SC_CARDCTL_OBERTHUR_KEY_RSA_CRT) {
+			sc_file_free(key_file);
 			return SC_ERROR_INVALID_ARGUMENTS;
+		}
 	
 		rsa_sbuf[5] = (key_file->id>>8) & 0xFF;
 		rsa_sbuf[6] = key_file->id & 0xFF;
@@ -1292,12 +1333,14 @@ auth_set_security_env(sc_card_t *card,
 			apdu.data = rsa_sbuf;
 		}
 		else {
+			sc_file_free(key_file);
 			sc_error(card->ctx, "Invalid crypto operation: %X\n", env->operation);
 			return SC_ERROR_NOT_SUPPORTED;
 		}
 	
 		break;
 	default:
+		sc_file_free(key_file);
 		sc_error(card->ctx, "Invalid crypto algorithm supplied.\n");
 		return SC_ERROR_NOT_SUPPORTED;
 	}
