@@ -235,9 +235,9 @@ pgp_read_blob(sc_card_t *card, struct blob *blob)
 	if (blob->info == NULL)
 		return blob->status;
 
-	card->ctx->suppress_errors++;
+	sc_ctx_suppress_errors_on(card->ctx);
 	r = blob->info->get_fn(card, blob->id, buffer, sizeof(buffer));
-	card->ctx->suppress_errors--;
+	sc_ctx_suppress_errors_off(card->ctx);
 
 	if (r < 0) {
 		blob->status = r;
@@ -614,8 +614,8 @@ pgp_compute_signature(sc_card_t *card, const u8 *data,
 	apdu.lc = data_len;
 	apdu.data = data;
 	apdu.datalen = data_len;
-	apdu.le = 256;
-	apdu.resp = out;
+	apdu.le      = outlen;
+	apdu.resp    = out;
 	apdu.resplen = outlen;
 
 	r = sc_transmit_apdu(card, &apdu);
@@ -689,7 +689,7 @@ pgp_decipher(sc_card_t *card, const u8 *in, size_t inlen,
 static int
 pgp_logout(sc_card_t *card)
 {
-	sc_error(card->ctx, "OpenPGP card: logout not supported\n");
+	sc_debug(card->ctx, "OpenPGP card: logout not supported\n");
 	return SC_ERROR_NOT_SUPPORTED;
 }
 
