@@ -68,15 +68,90 @@ static void set_string(char **strp, const char *value)
 	*strp = value ? strdup(value) : NULL;
 }
 
+#if 1
+/* XXX: temporary copy of the old pkcs15emu functions,
+ *      to be removed */
+static int sc_pkcs15emu_add_pin(sc_pkcs15_card_t *p15card,
+                const sc_pkcs15_id_t *id, const char *label,
+                const sc_path_t *path, int ref, int type,
+                unsigned int min_length,
+                unsigned int max_length,
+                int flags, int tries_left, const char pad_char, int obj_flags)
+{
+        sc_pkcs15_pin_info_t info;
+	sc_pkcs15_object_t   obj;
+
+	memset(&info, 0, sizeof(info));
+	memset(&obj,  0, sizeof(obj));
+
+        info.auth_id           = *id;
+        info.min_length        = min_length;
+        info.max_length        = max_length;
+        info.stored_length     = max_length;
+        info.type              = type;
+        info.reference         = ref;
+        info.flags             = flags;
+        info.tries_left        = tries_left;
+        info.magic             = SC_PKCS15_PIN_MAGIC;
+        info.pad_char          = pad_char;
+
+        if (path)
+                info.path = *path;
+        if (type == SC_PKCS15_PIN_TYPE_BCD)
+                info.stored_length /= 2;
+
+	strncpy(obj.label, label, SC_PKCS15_MAX_LABEL_SIZE-1);
+	obj.flags = obj_flags;
+
+        return sc_pkcs15emu_add_pin_obj(p15card, &obj, &info);
+}
+
+static int sc_pkcs15emu_add_prkey(sc_pkcs15_card_t *p15card,
+                const sc_pkcs15_id_t *id,
+                const char *label,
+                int type, unsigned int modulus_length, int usage,
+                const sc_path_t *path, int ref,
+                const sc_pkcs15_id_t *auth_id, int obj_flags)
+{
+        sc_pkcs15_prkey_info_t info;
+	sc_pkcs15_object_t     obj;
+
+	memset(&info, 0, sizeof(info));
+	memset(&obj,  0, sizeof(obj));
+
+        info.id                = *id;
+        info.modulus_length    = modulus_length;
+        info.usage             = usage;
+        info.native            = 1;
+        info.access_flags      = SC_PKCS15_PRKEY_ACCESS_SENSITIVE
+                                | SC_PKCS15_PRKEY_ACCESS_ALWAYSSENSITIVE
+                                | SC_PKCS15_PRKEY_ACCESS_NEVEREXTRACTABLE
+                                | SC_PKCS15_PRKEY_ACCESS_LOCAL;
+        info.key_reference     = ref;
+
+        if (path)
+                info.path = *path;
+
+	obj.flags = obj_flags;
+	strncpy(obj.label, label, SC_PKCS15_MAX_LABEL_SIZE-1);
+	if (auth_id != NULL)
+		obj.auth_id = *auth_id;
+
+        return sc_pkcs15emu_add_rsa_prkey(p15card, &obj, &info);
+}
+#endif
+
 static int sc_pkcs15emu_actalis_init(sc_pkcs15_card_t * p15card)
 {
 	sc_card_t *card = p15card->card;
 	sc_path_t path;
 	sc_pkcs15_id_t id, auth_id;
-	unsigned char serial[8];
+	unsigned char serial[9];
 	int flags;
-	int i, r;
+	int r;
 
+#ifdef HAVE_ZLIB_H
+	int i;
 	const char *certLabel[] = {
 		"User Non-repudiation Certificate",	/* "User Non-repudiation Certificate" */
 		"TSCA Certificate", 
@@ -84,6 +159,7 @@ static int sc_pkcs15emu_actalis_init(sc_pkcs15_card_t * p15card)
 	};	
 	const char *certPath[] =
 	    { "3F00300060006002", "3F00300060006003", "3F00300060006004" };
+#endif
 
 	const char *keyPath = "3F00300040000008";
 	const char *pinDfName = "05040200";
