@@ -134,7 +134,7 @@ static void print_dn(FILE *f, CK_LONG type, CK_VOID_PTR value, CK_ULONG size, CK
   print_generic(f, type, value, size, arg);
   if(size && value) {
     X509_NAME *name;
-    name = d2i_X509_NAME(NULL, (unsigned char **)&value, size);
+    name = d2i_X509_NAME(NULL, (const unsigned char **)&value, size);
     if(name) {
       BIO *bio = BIO_new(BIO_s_file());
       BIO_set_fp(bio, f, 0);
@@ -819,7 +819,8 @@ void print_mech_info(FILE *f, CK_MECHANISM_TYPE type,
     fprintf(f, "Unknown Mechanism (%08lx) : ", type);
   }
   fprintf(f, "min:%lu max:%lu flags:0x%lX ",
-	  minfo->ulMinKeySize, minfo->ulMaxKeySize, minfo->flags);
+	  (unsigned long) minfo->ulMinKeySize,
+	  (unsigned long) minfo->ulMaxKeySize, minfo->flags);
   printf("( %s%s%s%s%s%s%s%s%s%s%s%s%s%s)\n",
 #endif
 	 (minfo->flags & CKF_HW)                ? "Hardware " : "",
