@@ -184,6 +184,12 @@ struct sc_pkcs11_framework_ops {
  * PKCS#11 Slot (used to access card with specific framework data)
  */
 
+#ifndef _WIN32
+typedef unsigned long long sc_timestamp_t;
+#else
+typedef unsigned __int64   sc_timestamp_t;
+#endif
+
 struct sc_pkcs11_card {
 	int reader;
 	struct sc_card *card;
@@ -424,7 +430,7 @@ CK_RV sc_pkcs11_register_sign_and_hash_mechanism(struct sc_pkcs11_card *,
 /* Random generation functions */
 CK_RV sc_pkcs11_gen_keypair_soft(CK_KEY_TYPE keytype, CK_ULONG keybits,
 	struct sc_pkcs15_prkey *privkey, struct sc_pkcs15_pubkey *pubkey);
-CK_RV sc_pkcs11_verify_data(unsigned char *pubkey, int pubkey_len,
+CK_RV sc_pkcs11_verify_data(const unsigned char *pubkey, int pubkey_len,
 	CK_MECHANISM_TYPE mech, sc_pkcs11_operation_t *md,
 	unsigned char *inp, int inp_len,
 	unsigned char *signat, int signat_len);
