@@ -341,7 +341,7 @@ extern int	sc_pkcs15init_update_certificate(sc_pkcs15_card_t *p15card,
 				sc_profile_t *profile,
 				sc_pkcs15_object_t *obj,
 				const unsigned char *rawcert,
-				int certlen);
+				size_t certlen);
 
 extern int	sc_pkcs15init_create_file(struct sc_profile *,
 				struct sc_card *, struct sc_file *);
@@ -371,6 +371,8 @@ extern int	sc_pkcs15init_verify_key(struct sc_profile *, struct sc_card *,
 				sc_file_t *,  unsigned int, unsigned int);
 extern int	sc_pkcs15init_delete_by_path(struct sc_profile *,
 				struct sc_card *, const sc_path_t *path);
+extern int  sc_pkcs15init_update_any_df(sc_pkcs15_card_t *, sc_profile_t *, 
+			sc_pkcs15_df_t *, int);
 
 /* Erasing the card structure via rm -rf */
 extern int	sc_pkcs15init_erase_card_recursively(struct sc_card *,
@@ -391,7 +393,7 @@ extern struct sc_pkcs15init_operations *sc_pkcs15init_get_gpk_ops(void);
 extern struct sc_pkcs15init_operations *sc_pkcs15init_get_miocos_ops(void);
 extern struct sc_pkcs15init_operations *sc_pkcs15init_get_cryptoflex_ops(void);
 extern struct sc_pkcs15init_operations *sc_pkcs15init_get_cyberflex_ops(void);
-extern struct sc_pkcs15init_operations *sc_pkcs15init_get_etoken_ops(void);
+extern struct sc_pkcs15init_operations *sc_pkcs15init_get_cardos_ops(void);
 extern struct sc_pkcs15init_operations *sc_pkcs15init_get_jcop_ops(void);
 extern struct sc_pkcs15init_operations *sc_pkcs15init_get_starcos_ops(void);
 extern struct sc_pkcs15init_operations *sc_pkcs15init_get_oberthur_ops(void);
