@@ -93,9 +93,6 @@ static int	sc_pkcs15init_update_tokeninfo(struct sc_pkcs15_card *,
 			struct sc_profile *profile);
 static int	sc_pkcs15init_update_odf(struct sc_pkcs15_card *,
 			struct sc_profile *profile);
-static int  sc_pkcs15init_update_any_df(sc_pkcs15_card_t *p15card,
-			sc_profile_t *profile, 
-			sc_pkcs15_df_t *df, int is_new);
 static sc_pkcs15_object_t *sc_pkcs15init_new_object(int type, const char *label,
 	       		sc_pkcs15_id_t *auth_id, void *data);
 static int	sc_pkcs15init_add_object(struct sc_pkcs15_card *,
@@ -134,6 +131,14 @@ static int	sc_pkcs15init_read_info(sc_card_t *card, sc_profile_t *);
 static int	sc_pkcs15init_parse_info(sc_card_t *, const u8 *, size_t, sc_profile_t *);
 static int	sc_pkcs15init_write_info(sc_card_t *card, sc_profile_t *,
 			sc_pkcs15_object_t *pin_obj);
+static int	sc_pkcs15init_read_unusedspace(sc_pkcs15_card_t *);
+static int sc_pkcs15init_update_unusedspace(sc_pkcs15_card_t *, sc_profile_t *);
+static sc_pkcs15_unusedspace_t *merge_paths(sc_pkcs15_unusedspace_t *, const sc_path_t *);
+static int sc_pkcs15init_add_unusedspace(sc_pkcs15_card_t *,
+			sc_profile_t *, const sc_path_t *, const sc_pkcs15_id_t *);
+static int sc_pkcs15init_remove_unusedspace(sc_pkcs15_card_t *,
+			sc_profile_t *, const sc_path_t *);
+
 
 static struct profile_operations {
 	const char *name;
@@ -143,7 +148,8 @@ static struct profile_operations {
 	{ "miocos", (void *) sc_pkcs15init_get_miocos_ops },
 	{ "flex", (void *) sc_pkcs15init_get_cryptoflex_ops },
 	{ "cyberflex", (void *) sc_pkcs15init_get_cyberflex_ops },
-	{ "etoken", (void *) sc_pkcs15init_get_etoken_ops },
+	{ "cardos", (void *) sc_pkcs15init_get_cardos_ops },
+	{ "etoken", (void *) sc_pkcs15init_get_cardos_ops }, /* legacy */
 	{ "jcop", (void *) sc_pkcs15init_get_jcop_ops },
 	{ "starcos", (void *) sc_pkcs15init_get_starcos_ops },
 	{ "oberthur", (void *) sc_pkcs15init_get_oberthur_ops },
@@ -405,13 +411,13 @@ sc_pkcs15init_erase_card_recursively(sc_card_t *card,
 	if (sc_keycache_find_named_pin(NULL, SC_PKCS15INIT_SO_PIN) == -1) {
 		struct sc_pkcs15_card *p15card = NULL;
 
-		card->ctx->suppress_errors++;
+		sc_ctx_suppress_errors_on(card->ctx);
 		if (sc_pkcs15_bind(card, &p15card) >= 0) {
 			/* result of set_so_pin_from_card ignored */
 			set_so_pin_from_card(p15card, profile);
 			profile->p15_data = p15card;
 		}
-		card->ctx->suppress_errors--;
+		sc_ctx_suppress_errors_off(card->ctx);
 	}
 
 	/* Delete EF(DIR). This may not be very nice
@@ -428,9 +434,9 @@ sc_pkcs15init_erase_card_recursively(sc_card_t *card,
 			goto out;
 	}
 
-	card->ctx->suppress_errors++;
+	sc_ctx_suppress_errors_on(card->ctx);
 	r = sc_select_file(card, &df->path, &df);
-	card->ctx->suppress_errors--;
+	sc_ctx_suppress_errors_off(card->ctx);
 	if (r >= 0) {
 		r = sc_pkcs15init_rmdir(card, profile, df);
 		sc_file_free(df);
@@ -504,20 +510,24 @@ sc_pkcs15init_rmdir(sc_card_t *card, struct sc_profile *profile,
 	struct sc_path	path;
 	struct sc_file	*file, *parent;
 	int		r = 0, nfids;
-
-	sc_debug(card->ctx, "sc_pkcs15init_rmdir(%s)\n",
-			sc_print_path(&df->path));
+	char            pbuf[SC_MAX_PATH_STRING_SIZE];
 
 	if (df == NULL)
 		return SC_ERROR_INTERNAL;
+	r = sc_path_print(pbuf, sizeof(pbuf), &df->path);
+	if (r != SC_SUCCESS)
+		pbuf[0] = '\0';
+
+	sc_debug(card->ctx, "sc_pkcs15init_rmdir(%s)\n", pbuf);
+
 	if (df->type == SC_FILE_TYPE_DF) {
 		r = sc_pkcs15init_authenticate(profile, card, df,
 				SC_AC_OP_LIST_FILES);
 		if (r < 0)
 			return r;
-		card->ctx->suppress_errors++;
+		sc_ctx_suppress_errors_on(card->ctx);
 		r = sc_list_files(card, buffer, sizeof(buffer));
-		card->ctx->suppress_errors--;
+		sc_ctx_suppress_errors_off(card->ctx);
 		if (r < 0)
 			return r;
 
@@ -570,9 +580,9 @@ sc_pkcs15init_rmdir(sc_card_t *card, struct sc_profile *profile,
 	if (r < 0 && r != SC_ERROR_NOT_SUPPORTED)
 		return r;
 
-	card->ctx->suppress_errors++;
+	sc_ctx_suppress_errors_on(card->ctx);
 	r = sc_delete_file(card, &path);
-	card->ctx->suppress_errors--;
+	sc_ctx_suppress_errors_off(card->ctx);
 	return r;
 }
 
@@ -707,7 +717,7 @@ sc_pkcs15init_add_app(sc_card_t *card, struct sc_profile *profile,
 	 * all sorts of assumptions about DF and EF names, and
 	 * doesn't work if secure messaging is required for the
 	 * MF (which is the case with the GPK) */
-	app = (sc_app_info_t *) calloc(1, sizeof(*app));
+	app = (sc_app_info_t *)calloc(1, sizeof(*app));
 	if (app == NULL)
 		return SC_ERROR_OUT_OF_MEMORY;
 	app->path = p15spec->file_app->path;
@@ -748,17 +758,183 @@ sc_pkcs15init_add_app(sc_card_t *card, struct sc_profile *profile,
 				SC_PKCS15_AODF, NULL);
 	}
 
-	if (r >= 0)
+	if (r >= 0) {
 		r = sc_pkcs15init_update_dir(p15spec, profile, app);
-	if (r >= 0)
-		r = sc_pkcs15init_update_tokeninfo(p15spec, profile);
+		if (r >= 0)
+			r = sc_pkcs15init_update_tokeninfo(p15spec, profile);
+		/* FIXME: what to do if sc_pkcs15init_update_dir failed? */
+	} else {
+		free(app); /* unused */
+	}
 
-	card->ctx->suppress_errors++;
+	sc_ctx_suppress_errors_on(card->ctx);
 	sc_pkcs15init_write_info(card, profile, pin_obj);
-	card->ctx->suppress_errors--;
+	sc_ctx_suppress_errors_off(card->ctx);
 	return r;
 }
 
+/* Read the EF(UnusedSpace) file */
+static int sc_pkcs15init_read_unusedspace(sc_pkcs15_card_t *p15card)
+{
+	sc_path_t path;
+	u8 *buf = NULL;
+	size_t buf_len;
+	int r;
+
+	/* Check if EF(UnusedSpace) file is already read */
+	if (p15card->unusedspace_read)
+		return 0;
+
+	if (p15card->file_unusedspace != NULL)
+		path = p15card->file_unusedspace->path;
+	else {
+		path = p15card->file_app->path;
+		sc_append_path_id(&path, (const u8 *) "\x50\x33", 2);
+		path.count = -1;
+	}
+
+	r = sc_pkcs15_read_file(p15card, &path, &buf, &buf_len, NULL);
+	if (r < 0) {
+		if (r == SC_ERROR_FILE_NOT_FOUND)
+			r = 0;
+		goto err;
+	}
+
+	r = sc_pkcs15_parse_unusedspace(buf, buf_len, p15card);
+
+err:
+	if (buf != NULL)
+		free(buf);
+	return r;
+}
+
+/* Update the EF(UnusedSpace) file */
+static int sc_pkcs15init_update_unusedspace(sc_pkcs15_card_t *p15card,
+	sc_profile_t *profile)
+{
+	u8 *buf = NULL;
+	size_t buf_len;
+	sc_file_t *file = NULL;
+	int r;
+
+	/* Make sure we've read the EF(UnusedSpace) file first */
+	r = sc_pkcs15init_read_unusedspace(p15card);
+	if (r < 0)
+		return r;
+	if (p15card->unusedspace_list == NULL)
+		return 0;
+
+	r = sc_profile_get_file(profile, "PKCS15-UnusedSpace", &file);
+	if (r < 0)
+		return r;
+
+	r = sc_pkcs15_encode_unusedspace(p15card->card->ctx, p15card, &buf, &buf_len);
+	if (r < 0)
+		goto err;
+
+	r = sc_pkcs15init_update_file(profile, p15card->card,
+		file, buf, buf_len);
+
+err:
+	if (buf != NULL)
+		free(buf);
+	if (file != NULL)
+		sc_file_free(file);
+	return r;
+}
+
+/* Called by sc_pkcs15init_add_unusedspace(), to try to merge path
+ * with one of the paths in us, so one large path can be made */
+static sc_pkcs15_unusedspace_t *merge_paths(sc_pkcs15_unusedspace_t *us,
+	const sc_path_t *path)
+{
+	for ( ; us != NULL; us = us->next) {
+		sc_path_t *old = &us->path;
+		if (!sc_compare_path(path, old))
+			continue;
+		if (old->index + old->count == path->index) {
+			old->count += path->count;
+			return us;
+		}
+		if (path->index + path->count == old->index) {
+			old->index = path->index;
+			old->count += path->count;
+			return us;
+		}
+	}
+	return NULL; /* Couldn't merge */
+}
+
+/* Add a path to the EF(UnusedSpace) file. This is done when (part of) the
+ * file where the path points to is no longer used (i.e. the pkcs15 object
+ * inside has been "deleted"). */
+static int sc_pkcs15init_add_unusedspace(sc_pkcs15_card_t *p15card,
+	sc_profile_t *profile, const sc_path_t *path, const sc_pkcs15_id_t *auth_id)
+{
+	sc_pkcs15_unusedspace_t *us;
+	int r = 0;
+
+	if (path->count == -1)
+		return SC_ERROR_INVALID_ARGUMENTS;
+
+	/* Make sure we've read the EF(UnusedSpace) file first */
+	r = sc_pkcs15init_read_unusedspace(p15card);
+	if (r < 0)
+		return r;
+
+	/* See if we can merge this new entry with one that already exists */
+	us = merge_paths(p15card->unusedspace_list, path);
+	if (us == NULL)
+		sc_pkcs15_add_unusedspace(p15card, path, auth_id);
+	else {
+		/* So we could merge it. But now the path pointed to by us
+		* might be mergeable with another path further on in the list */
+		if (merge_paths(us->next, &us->path))
+			sc_pkcs15_remove_unusedspace(p15card, us);
+	}
+
+	return sc_pkcs15init_update_unusedspace(p15card, profile);
+}
+
+/* Remove some space from the EF(UnusedSpace) file. This is done when you want
+ * to use the space  for a certificate, data, ... */
+static int sc_pkcs15init_remove_unusedspace(sc_pkcs15_card_t *p15card,
+	sc_profile_t *profile, const sc_path_t *path)
+{
+	sc_pkcs15_unusedspace_t *us;
+	int ok = 0;
+	int r = 0;
+
+	/* Make sure we've read the EF(UnusedSpace) file first */
+	r = sc_pkcs15init_read_unusedspace(p15card);
+	if (r < 0)
+		return r;
+
+	/* Search in the EF(UnusedSpace) for a path where the required
+	 * space (referred to by 'path') can be subtracted from  */
+	for (us = p15card->unusedspace_list; us != NULL && !ok; us = us->next) {
+		sc_path_t *old = &us->path;
+		if (!sc_compare_path(path, old) || old->count < path->count)
+				continue;
+		if (old->index == path->index) {
+			old->index += path->count;
+			old->count -= path->count;
+			ok = 1;
+		}
+		else if (old->index + old->count == path->index + path->count) {
+			old->count -= path->count;
+			ok = 1;
+		}
+		if (old->count == 0)
+			sc_pkcs15_remove_unusedspace(p15card, us);
+	}
+
+	if (!ok)
+		return SC_ERROR_OBJECT_NOT_FOUND; /* the space couldn't be found */
+
+	return sc_pkcs15init_update_unusedspace(p15card, profile);
+}
+
 /*
  * Store a PIN/PUK pair
  */
@@ -778,7 +954,7 @@ sc_pkcs15init_store_pin(struct sc_pkcs15_card *p15card,
 		unsigned int	n;
 
 		args->auth_id.len = 1;
-		card->ctx->suppress_errors++;
+		sc_ctx_suppress_errors_on(card->ctx);
 		for (n = 1, r = 0; n < 256; n++) {
 			args->auth_id.value[0] = n;
 			r = sc_pkcs15_find_pin_by_auth_id(p15card,
@@ -786,7 +962,7 @@ sc_pkcs15init_store_pin(struct sc_pkcs15_card *p15card,
 			if (r == SC_ERROR_OBJECT_NOT_FOUND)
 				break;
 		}
-		card->ctx->suppress_errors--;
+		sc_ctx_suppress_errors_off(card->ctx);
 		if (r != SC_ERROR_OBJECT_NOT_FOUND) {
 			sc_error(card->ctx, "No auth_id specified for new PIN");
 			return SC_ERROR_INVALID_ARGUMENTS;
@@ -795,9 +971,9 @@ sc_pkcs15init_store_pin(struct sc_pkcs15_card *p15card,
 		struct sc_pkcs15_object *dummy;
 
 		/* Make sure we don't get duplicate PIN IDs */
-		card->ctx->suppress_errors++;
+		sc_ctx_suppress_errors_on(card->ctx);
 		r = sc_pkcs15_find_pin_by_auth_id(p15card, &args->auth_id, &dummy);
-		card->ctx->suppress_errors--;
+		sc_ctx_suppress_errors_off(card->ctx);
 		if (r != SC_ERROR_OBJECT_NOT_FOUND) {
 			sc_error(card->ctx, "There already is a PIN with this ID.");
 			return SC_ERROR_INVALID_ARGUMENTS;
@@ -1503,25 +1679,33 @@ sc_pkcs15init_store_certificate(struct sc_pkcs15_card *p15card,
 	if ((label = args->label) == NULL)
 		label = "Certificate";
 
+	/* Set the SO PIN reference from card */
+	if ((r = set_so_pin_from_card(p15card, profile)) < 0)
+		return r;
+
 	/* Select an ID if the user didn't specify one, otherwise
 	 * make sure it's unique */
 	if ((r = select_id(p15card, SC_PKCS15_TYPE_CERT, &args->id, NULL, NULL, NULL)) < 0)
 		return r;
 
-	/* If there is a private key corresponding to the ID given
-	 * by the user, make sure $PIN references the pin protecting
-	 * this key
-	 */
-	if (args->id.len != 0
-	 && profile->protect_certificates
-	 && sc_pkcs15_find_prkey_by_id(p15card, &args->id, &object) == 0) {
-		r = set_user_pin_from_authid(p15card, profile, &object->auth_id);
-		if (r < 0) {
-			sc_error(p15card->card->ctx,
-				      "Failed to assign user pin reference "
-				      "(copied from private key auth_id)\n");
-			return r;
+	if (profile->protect_certificates) {
+		/* If there is a private key corresponding to the ID given
+		 * by the user, make sure $PIN references the pin protecting
+		 * this key
+		 */
+		r = -1;
+		if (args->id.len != 0
+		 && sc_pkcs15_find_prkey_by_id(p15card, &args->id, &object) == 0) {
+			r = set_user_pin_from_authid(p15card, profile, &object->auth_id);
+			if (r < 0) {
+				sc_error(p15card->card->ctx,
+					      "Failed to assign user pin reference "
+					      "(copied from private key auth_id)\n");
+				return r;
+			}
 		}
+		if (r == -1) /* User pin ref not yet set */
+			set_user_pin_from_authid(p15card, profile, NULL);
 	}
 
 	object = sc_pkcs15init_new_object(SC_PKCS15_TYPE_CERT_X509, label, NULL, NULL);
@@ -2083,6 +2267,7 @@ select_object_path(sc_pkcs15_card_t *p15card, sc_profile_t *profile,
 	sc_file_t	*file;
 	const char	*name;
 	int		r;
+	char		pbuf[SC_MAX_PATH_STRING_SIZE];
 
 	/* For cards with a pin-domain profile, we need
 	 * to put the key below the DF of the specified PIN */
@@ -2115,9 +2300,13 @@ select_object_path(sc_pkcs15_card_t *p15card, sc_profile_t *profile,
 		return 0;
 	}
 
+	r = sc_path_print(pbuf, sizeof(pbuf), path);
+	if (r != SC_SUCCESS)
+		pbuf[0] = '\0';
+
 	sc_debug(p15card->card->ctx,
-		"key-domain.%s @%s (auth_id.len=%d)\n",
-		name, sc_print_path(path), obj->auth_id.len);
+		"key-domain.%s @%s (auth_id.len=%d)\n", 
+		name, pbuf, obj->auth_id.len);
 	r = sc_profile_instantiate_template(profile,
 					"key-domain", path, 
 					name, obj_id, &file);
@@ -2147,9 +2336,9 @@ sc_pkcs15init_update_dir(struct sc_pkcs15_card *p15card,
 		struct sc_file	*dir_file;
 		struct sc_path	path;
 
-		card->ctx->suppress_errors++;
+		sc_ctx_suppress_errors_on(card->ctx);
 		r = sc_enum_apps(card);
-		card->ctx->suppress_errors--;
+		sc_ctx_suppress_errors_off(card->ctx);
 
 		if (r != SC_ERROR_FILE_NOT_FOUND)
 			break;
@@ -2210,16 +2399,28 @@ static int sc_pkcs15init_update_tokeninfo(struct sc_pkcs15_card *p15card,
 		struct sc_profile *profile)
 {
 	struct sc_card	*card = p15card->card;
+	sc_pkcs15_tokeninfo_t tokeninfo;
 	u8		*buf = NULL;
 	size_t		size;
 	int		r;
 
 	/* set lastUpdate field */
+	if (p15card->last_update != NULL)
+		free(p15card->last_update);
 	p15card->last_update = get_generalized_time(card->ctx);
 	if (p15card->last_update == NULL)
 		return SC_ERROR_INTERNAL;
 
-	r = sc_pkcs15_encode_tokeninfo(card->ctx, p15card, &buf, &size);
+	/* create a temporary tokeninfo structure */
+	tokeninfo.version = p15card->version;
+	tokeninfo.flags = p15card->flags;
+	tokeninfo.label = p15card->label;
+	tokeninfo.serial_number = p15card->serial_number;
+	tokeninfo.manufacturer_id = p15card->manufacturer_id;
+	tokeninfo.last_update = p15card->last_update;
+	tokeninfo.preferred_language = p15card->preferred_language;
+
+	r = sc_pkcs15_encode_tokeninfo(card->ctx, &tokeninfo, &buf, &size);
 	if (r >= 0)
 		r = sc_pkcs15init_update_file(profile, card,
 			       p15card->file_tokeninfo, buf, size);
@@ -2250,7 +2451,7 @@ sc_pkcs15init_update_odf(struct sc_pkcs15_card *p15card,
 /*
  * Update any PKCS15 DF file (except ODF and DIR)
  */
-static int
+int
 sc_pkcs15init_update_any_df(sc_pkcs15_card_t *p15card, 
 		sc_profile_t *profile,
 		sc_pkcs15_df_t *df,
@@ -2482,7 +2683,7 @@ sc_pkcs15init_new_object(int type, const char *label, sc_pkcs15_id_t *auth_id, v
 	}
 
 	if (label)
-		strncpy(object->label, label, sizeof(object->label));
+		strncpy(object->label, label, sizeof(object->label)-1);
 	if (auth_id)
 		object->auth_id = *auth_id;
 
@@ -2561,9 +2762,6 @@ int sc_pkcs15init_delete_object(sc_pkcs15_card_t *p15card,
 	struct sc_pkcs15_df *df;
 	int r;
 
-	if (profile->ops->delete_object == NULL)
-		return SC_ERROR_NOT_SUPPORTED;
-
 	switch(obj->type & SC_PKCS15_TYPE_CLASS_MASK)
 	{
 	case SC_PKCS15_TYPE_PUBKEY:
@@ -2586,21 +2784,35 @@ int sc_pkcs15init_delete_object(sc_pkcs15_card_t *p15card,
 	if ((r = set_so_pin_from_card(p15card, profile)) < 0)
 		return r;
 
-	r = profile->ops->delete_object(profile, p15card->card,
-		obj->type, obj->data, &path);
-	if (r < 0) {
-		sc_error(p15card->card->ctx, "ops->delete_object() failed: %d", r);
-		return r;
+	/* If there's a card-specific way to delete objects, use it.
+	 * Otherwise, just set its label to "deleted" to indicate
+	 * that we can re-used it when we have to make a next
+	 * object in the future. */
+	if (profile->ops->delete_object != NULL) {
+		r = profile->ops->delete_object(profile, p15card->card,
+			obj->type, obj->data, &path);
+		if (r < 0) {
+			sc_error(p15card->card->ctx, "ops->delete_object() failed: %d", r);
+			return r;
+		}
+
+		/* Get the DF we're part of. If there's no DF, fine, we haven't
+		 * been added yet. */
+		if ((df = obj->df) != NULL) {
+			/* Unlink the object and update the DF */
+			sc_pkcs15_remove_object(p15card, obj);
+			r = sc_pkcs15init_update_any_df(p15card, profile, df, 0);
+		}
+	}
+	else {
+		/* Get the DF we're part of. If there's no DF, fine, we haven't
+		 * been added yet. */
+		if ((df = obj->df) != NULL) {
+			/*Change the label into "deleted" and update the DF */
+			strcpy(obj->label, "deleted");
+			r = sc_pkcs15init_update_any_df(p15card, profile, df, 0);
+		}
 	}
-
-	/* Get the DF we're part of. If there's no DF, fine, we haven't
-	 * been added yet. */
-	if ((df = obj->df) == NULL)
-		return 0;
-
-	/* Unlink the object and update the DF */
-	sc_pkcs15_remove_object(p15card, obj);
-	r = sc_pkcs15init_update_any_df(p15card, profile, df, 0);
 
 	/* mark card as dirty */
 	profile->dirty = 1;
@@ -2612,7 +2824,7 @@ int
 sc_pkcs15init_update_certificate(sc_pkcs15_card_t *p15card,
 	sc_profile_t *profile,
 	sc_pkcs15_object_t *obj,
-	const unsigned char *rawcert, int certlen)
+	const unsigned char *rawcert, size_t certlen)
 {
 	sc_file_t *file = NULL, *parent = NULL;
 	sc_path_t *path = &((sc_pkcs15_cert_info_t *)obj->data)->path;
@@ -2898,7 +3110,9 @@ set_so_pin_from_card(struct sc_pkcs15_card *p15card, struct sc_profile *profile)
 
 /*
  * If the user specified an auth_id, select the corresponding
- * PIN entry and set the reference data
+ * PIN entry and set the reference data.
+ * If auth_id is NULL, then get the first user PIN found, this
+ * is usefull for the 'onepin' profile option.
  */
 static int
 set_user_pin_from_authid(struct sc_pkcs15_card *p15card,
@@ -2909,6 +3123,23 @@ set_user_pin_from_authid(struct sc_pkcs15_card *p15card,
 	struct sc_pkcs15_object	*objp;
 	int		r;
 
+	if (auth_id == NULL) {
+		int i;
+		struct sc_pkcs15_object	*p15objects[5];
+		r = sc_pkcs15_get_objects(p15card, SC_PKCS15_TYPE_AUTH_PIN, p15objects, 5);
+		if (r < 0)
+			return r;
+		for (i = 0; i < r; i++) {
+			sc_pkcs15_pin_info_t *pininfo = (sc_pkcs15_pin_info_t *) p15objects[i]->data;
+			if (!(pininfo->flags & SC_PKCS15_PIN_FLAG_SO_PIN)) {
+				auth_id = &pininfo->auth_id;
+				break;
+			}
+		}
+		if (i >= r)
+			return SC_ERROR_OBJECT_NOT_FOUND;
+	}
+
 	if (auth_id->len == 0)
 		return 0;
 
@@ -2960,10 +3191,14 @@ sc_pkcs15init_authenticate(struct sc_profile *pro, sc_card_t *card,
 {
 	const sc_acl_entry_t *acl;
 	sc_file_t *file_tmp = NULL;
-	int		r = 0;
+	int  r = 0;
+	char pbuf[SC_MAX_PATH_STRING_SIZE];     
 
-	sc_debug(card->ctx, "path=%s, op=%u\n",
-				sc_print_path(&file->path), op);
+	r = sc_path_print(pbuf, sizeof(pbuf), &file->path);
+	if (r != SC_SUCCESS)
+		pbuf[0] = '\0';
+
+	sc_debug(card->ctx, "path=%s, op=%u\n", pbuf, op);
 
 	if (card->caps & SC_CARD_CAP_USE_FCI_AC) {
 		if ((r = sc_select_file(card, &file->path, &file_tmp)) < 0)
@@ -3008,16 +3243,22 @@ do_select_parent(struct sc_profile *pro, sc_card_t *card,
 
 	/* Select the parent DF. */
 	*parent = NULL;
-	card->ctx->suppress_errors++;
+	sc_ctx_suppress_errors_on(card->ctx);
 	r = sc_select_file(card, &path, parent);
-	card->ctx->suppress_errors--;
+	sc_ctx_suppress_errors_off(card->ctx);
 	/* If DF doesn't exist, create it (unless it's the MF,
 	 * but then something's badly broken anyway :-) */
 	if (r == SC_ERROR_FILE_NOT_FOUND && path.len != 2) {
 		r = sc_profile_get_file_by_path(pro, &path, parent);
 		if (r < 0) {
-			sc_error(card->ctx, "profile doesn't define a DF %s",
-					sc_print_path(&path));
+			char pbuf[SC_MAX_PATH_STRING_SIZE];
+
+			r = sc_path_print(pbuf, sizeof(pbuf), &path);
+			if (r != SC_SUCCESS)
+				pbuf[0] = '\0';
+
+			sc_error(card->ctx,
+				"profile doesn't define a DF %s", pbuf);
 			return r;
 		}
 		if (!(r = sc_pkcs15init_create_file(pro, card, *parent)))
@@ -3028,8 +3269,14 @@ do_select_parent(struct sc_profile *pro, sc_card_t *card,
 		sc_file_free(*parent);
 		r = sc_profile_get_file_by_path(pro, &path, parent);
 		if (r < 0) {
-			sc_error(card->ctx, "profile doesn't define a DF %s",
-					sc_print_path(&path));
+			char pbuf[SC_MAX_PATH_STRING_SIZE];
+
+			r = sc_path_print(pbuf, sizeof(pbuf), &path);
+			if (r != SC_SUCCESS)
+				pbuf[0] = '\0';
+
+			sc_error(card->ctx,
+				"profile doesn't define a DF %s", pbuf);
 			return r;
 		}
 	}
@@ -3072,13 +3319,16 @@ sc_pkcs15init_update_file(struct sc_profile *profile, sc_card_t *card,
 	struct sc_file	*info = NULL;
 	void		*copy = NULL;
 	int		r, need_to_zap = 0;
+	char		pbuf[SC_MAX_PATH_STRING_SIZE];
 
-	sc_debug(card->ctx, "called, path=%s, %u bytes\n",
-			sc_print_path(&file->path), datalen);
+	r = sc_path_print(pbuf, sizeof(pbuf), &file->path);
+	if (r != SC_SUCCESS)
+		pbuf[0] = '\0';
+	sc_debug(card->ctx, "called, path=%s, %u bytes\n", pbuf, datalen);
 
-	card->ctx->suppress_errors++;
+	sc_ctx_suppress_errors_on(card->ctx);
 	if ((r = sc_select_file(card, &file->path, &info)) < 0) {
-		card->ctx->suppress_errors--;
+		sc_ctx_suppress_errors_off(card->ctx);
 		/* Create file if it doesn't exist */
 		if (file->size < datalen)
 			file->size = datalen;
@@ -3087,16 +3337,21 @@ sc_pkcs15init_update_file(struct sc_profile *profile, sc_card_t *card,
 		 || (r = sc_select_file(card, &file->path, &info)) < 0)
 			return r;
 	} else {
-		card->ctx->suppress_errors--;
+		sc_ctx_suppress_errors_off(card->ctx);
 		need_to_zap = 1;
 	}
 
 	if (info->size < datalen) {
+		char pbuf[SC_MAX_PATH_STRING_SIZE];
+
+		r = sc_path_print(pbuf, sizeof(pbuf), &file->path);
+		if (r != SC_SUCCESS)
+			pbuf[0] = '\0';
+
 		sc_error(card->ctx,
-			      "File %s too small (require %u, have %u) - "
-			      "please increase size in profile",
-			      sc_print_path(&file->path),
-			      datalen, info->size);
+			"File %s too small (require %u, have %u) - "
+			"please increase size in profile", pbuf,
+			datalen, info->size);
 		sc_file_free(info);
 		return SC_ERROR_TOO_MANY_OBJECTS;
 	} else if (info->size > datalen && need_to_zap) {
@@ -3340,7 +3595,7 @@ sc_pkcs15init_read_info(sc_card_t *card, sc_profile_t *profile)
 	size_t		len = 0;
 	int		r;
 
-	card->ctx->suppress_errors++;
+	sc_ctx_suppress_errors_on(card->ctx);
 	sc_format_path(OPENSC_INFO_FILEPATH, &path);
 	if ((r = sc_select_file(card, &path, &file)) >= 0) {
 		len = file->size;
@@ -3352,7 +3607,7 @@ sc_pkcs15init_read_info(sc_card_t *card, sc_profile_t *profile)
 	} else {
 		r = 0;
 	}
-	card->ctx->suppress_errors--;
+	sc_ctx_suppress_errors_off(card->ctx);
 
 	if (r >= 0)
 		r = sc_pkcs15init_parse_info(card, mem, len, profile);
