@@ -1,5 +1,5 @@
 /*
- * $Id: parse.c 1742 2004-01-22 12:37:26Z aet $
+ * $Id: parse.c 2700 2005-12-05 21:21:02Z aj $
  *
  * Copyright (C) 2002
  *  Antti Tapaninen <aet@cc.hut.fi>
@@ -152,6 +152,8 @@ scconf_item *scconf_item_add(scconf_context * config, scconf_block * block, scco
 		parser.current_item->value.comment = strdup((char *) data);
 		break;
 	case SCCONF_ITEM_TYPE_BLOCK:
+		if (!dst)
+			return NULL;
 		dst->parent = parser.block;
 		parser.current_item->value.block = dst;
 		scconf_list_destroy(parser.name);
@@ -192,8 +194,11 @@ scconf_block *scconf_block_add(scconf_context * config, scconf_block * block, co
 {
 	scconf_parser parser;
 
+	if (!config)
+		return NULL;
+
 	memset(&parser, 0, sizeof(scconf_parser));
-	parser.config = config ? config : NULL;
+	parser.config = config;
 	parser.key = key ? strdup(key) : NULL;
 	parser.block = block ? block : config->root;
 	scconf_list_copy(name, &parser.name);
