@@ -783,14 +783,14 @@ static int login(CK_SESSION_HANDLE session, int need_to_be_so)
 void
 init_token(CK_SLOT_ID slot)
 {
-	char token_label[33];
+	unsigned char token_label[33];
 	char new_buf[21], *new_pin = NULL;
 	CK_TOKEN_INFO	info;
 	CK_RV rv;
 
 	if (!opt_object_label)
 		fatal("The token label must be specified using --label\n");
-	snprintf(token_label, sizeof (token_label), "%-32.32s",
+	snprintf((char *) token_label, sizeof (token_label), "%-32.32s",
 			opt_object_label);
 
 	get_token_info(slot, &info);
@@ -1525,23 +1525,31 @@ get##ATTR(CK_SESSION_HANDLE sess, CK_OBJECT_HANDLE obj, CK_ULONG_PTR pulCount) \
  * Define attribute accessors
  */
 ATTR_METHOD(CLASS, CK_OBJECT_CLASS);
+#if 0
 ATTR_METHOD(TOKEN, CK_BBOOL);
 ATTR_METHOD(LOCAL, CK_BBOOL);
 ATTR_METHOD(SENSITIVE, CK_BBOOL);
 ATTR_METHOD(ALWAYS_SENSITIVE, CK_BBOOL);
 ATTR_METHOD(NEVER_EXTRACTABLE, CK_BBOOL);
+#endif
 ATTR_METHOD(PRIVATE, CK_BBOOL);
 ATTR_METHOD(MODIFIABLE, CK_BBOOL);
 ATTR_METHOD(ENCRYPT, CK_BBOOL);
 ATTR_METHOD(DECRYPT, CK_BBOOL);
 ATTR_METHOD(SIGN, CK_BBOOL);
+#if 0
 ATTR_METHOD(SIGN_RECOVER, CK_BBOOL);
+#endif
 ATTR_METHOD(VERIFY, CK_BBOOL);
+#if 0
 ATTR_METHOD(VERIFY_RECOVER, CK_BBOOL);
+#endif
 ATTR_METHOD(WRAP, CK_BBOOL);
 ATTR_METHOD(UNWRAP, CK_BBOOL);
+#if 0
 ATTR_METHOD(DERIVE, CK_BBOOL);
 ATTR_METHOD(EXTRACTABLE, CK_BBOOL);
+#endif
 ATTR_METHOD(KEY_TYPE, CK_KEY_TYPE);
 ATTR_METHOD(CERTIFICATE_TYPE, CK_CERTIFICATE_TYPE);
 ATTR_METHOD(MODULUS_BITS, CK_ULONG);
@@ -1589,12 +1597,14 @@ show_key(CK_SESSION_HANDLE sess, CK_OBJECT_HANDLE obj, int pub)
 	switch (key_type) {
 	case CKK_RSA:
 		if (pub)
-			printf("; RSA %lu bits\n", getMODULUS_BITS(sess, obj));
+			printf("; RSA %lu bits\n",
+				(unsigned long) getMODULUS_BITS(sess, obj));
 		else
 			printf("; RSA \n");
 		break;
 	default:
-		printf("; unknown key algorithm %lu\n", key_type);
+		printf("; unknown key algorithm %lu\n",
+				(unsigned long) key_type);
 		break;
 	}
 
@@ -1763,7 +1773,7 @@ get_mechanisms(CK_SLOT_ID slot,
 	if (rv != CKR_OK)
 		p11_fatal("C_GetMechanismList", rv);
 
-	if (flags != -1) {
+	if (flags != (CK_FLAGS)-1) {
 		CK_MECHANISM_TYPE *mechs = *pList;
 		CK_MECHANISM_INFO info;
 
@@ -1815,7 +1825,7 @@ read_object(CK_SLOT_ID slot, CK_SESSION_HANDLE session)
 	}
 
 	if (opt_application_id != NULL)   {
-		parse_application_id(&oid, opt_application_id);
+		sc_format_oid(&oid, opt_application_id);
 		FILL_ATTR(attrs[nn_attrs], CKA_OBJECT_ID,
 				(unsigned char *)oid.value, sizeof(oid.value));
 		nn_attrs++;
@@ -2716,7 +2726,7 @@ encrypt_decrypt(CK_SLOT_ID slot, CK_SESSION_HANDLE session,
 	if (pkey == NULL)
 		return 0;
 
-	if (EVP_PKEY_size(pkey) > sizeof(encrypted)) {
+	if (EVP_PKEY_size(pkey) > (int)sizeof(encrypted)) {
 		printf("Ciphertext buffer too small\n");
 		EVP_PKEY_free(pkey);
 		return 0;
@@ -3156,7 +3166,7 @@ p11_slot_info_flags(CK_FLAGS value)
 		{ CKF_TOKEN_PRESENT, "token present" },
 		{ CKF_REMOVABLE_DEVICE, "removable device" },
 		{ CKF_HW_SLOT, "hardware slot" },
-		{ 0 }
+		{ 0, NULL }
 	};
 
 	return p11_flag_names(slot_flags, value);
@@ -3172,7 +3182,7 @@ p11_token_info_flags(CK_FLAGS value)
 		{ CKF_USER_PIN_INITIALIZED, "PIN initialized" },
 		{ CKF_PROTECTED_AUTHENTICATION_PATH, "PIN pad present" },
 		{ CKF_TOKEN_INITIALIZED, "token initialized" },
-		{ 0 }
+		{ 0, NULL }
 	};
 
 	return p11_flag_names(slot_flags, value);
@@ -3268,181 +3278,181 @@ int hex_to_bin(const char *in, unsigned char *out, size_t *outlen)
 }
 
 static struct mech_info	p11_mechanisms[] = {
-      { CKM_RSA_PKCS_KEY_PAIR_GEN,		"RSA-PKCS-KEY-PAIR-GEN" },
-      { CKM_RSA_PKCS,		"RSA-PKCS" },
-      { CKM_RSA_9796,		"RSA-9796" },
-      { CKM_RSA_X_509,		"RSA-X-509" },
-      { CKM_MD2_RSA_PKCS,	"MD2-RSA-PKCS" },
-      { CKM_MD5_RSA_PKCS,	"MD5-RSA-PKCS",		"rsa-md5" },
+      { CKM_RSA_PKCS_KEY_PAIR_GEN,	"RSA-PKCS-KEY-PAIR-GEN", NULL },
+      { CKM_RSA_PKCS,		"RSA-PKCS",	NULL },
+      { CKM_RSA_9796,		"RSA-9796",	NULL },
+      { CKM_RSA_X_509,		"RSA-X-509",	NULL },
+      { CKM_MD2_RSA_PKCS,	"MD2-RSA-PKCS", 	NULL },
+      { CKM_MD5_RSA_PKCS,	"MD5-RSA-PKCS", 	"rsa-md5" },
       { CKM_SHA1_RSA_PKCS,	"SHA1-RSA-PKCS",	"rsa-sha1" },
-      { CKM_RIPEMD128_RSA_PKCS,	"RIPEMD128-RSA-PKCS" },
+      { CKM_RIPEMD128_RSA_PKCS,	"RIPEMD128-RSA-PKCS",	NULL },
       { CKM_RIPEMD160_RSA_PKCS,	"RIPEMD160-RSA-PKCS",	"rsa-ripemd160" },
-      { CKM_RSA_PKCS_OAEP,	"RSA-PKCS-OAEP" },
-      { CKM_RSA_X9_31_KEY_PAIR_GEN,"RSA-X9-31-KEY-PAIR-GEN" },
-      { CKM_RSA_X9_31,		"RSA-X9-31" },
-      { CKM_SHA1_RSA_X9_31,	"SHA1-RSA-X9-31" },
-      { CKM_RSA_PKCS_PSS,	"RSA-PKCS-PSS" },
-      { CKM_SHA1_RSA_PKCS_PSS,	"SHA1-RSA-PKCS-PSS" },
-      { CKM_DSA_KEY_PAIR_GEN,	"DSA-KEY-PAIR-GEN" },
-      { CKM_DSA,		"DSA" },
-      { CKM_DSA_SHA1,		"DSA-SHA1" },
-      { CKM_DH_PKCS_KEY_PAIR_GEN,"DH-PKCS-KEY-PAIR-GEN" },
-      { CKM_DH_PKCS_DERIVE,	"DH-PKCS-DERIVE" },
-      { CKM_X9_42_DH_KEY_PAIR_GEN,"X9-42-DH-KEY-PAIR-GEN" },
-      { CKM_X9_42_DH_DERIVE,	"X9-42-DH-DERIVE" },
-      { CKM_X9_42_DH_HYBRID_DERIVE,"X9-42-DH-HYBRID-DERIVE" },
-      { CKM_X9_42_MQV_DERIVE,	"X9-42-MQV-DERIVE" },
-      { CKM_RC2_KEY_GEN,	"RC2-KEY-GEN" },
-      { CKM_RC2_ECB,		"RC2-ECB" },
-      { CKM_RC2_CBC,		"RC2-CBC" },
-      { CKM_RC2_MAC,		"RC2-MAC" },
-      { CKM_RC2_MAC_GENERAL,	"RC2-MAC-GENERAL" },
-      { CKM_RC2_CBC_PAD,	"RC2-CBC-PAD" },
-      { CKM_RC4_KEY_GEN,	"RC4-KEY-GEN" },
-      { CKM_RC4,		"RC4" },
-      { CKM_DES_KEY_GEN,	"DES-KEY-GEN" },
-      { CKM_DES_ECB,		"DES-ECB" },
-      { CKM_DES_CBC,		"DES-CBC" },
-      { CKM_DES_MAC,		"DES-MAC" },
-      { CKM_DES_MAC_GENERAL,	"DES-MAC-GENERAL" },
-      { CKM_DES_CBC_PAD,	"DES-CBC-PAD" },
-      { CKM_DES2_KEY_GEN,	"DES2-KEY-GEN" },
-      { CKM_DES3_KEY_GEN,	"DES3-KEY-GEN" },
-      { CKM_DES3_ECB,		"DES3-ECB" },
-      { CKM_DES3_CBC,		"DES3-CBC" },
-      { CKM_DES3_MAC,		"DES3-MAC" },
-      { CKM_DES3_MAC_GENERAL,	"DES3-MAC-GENERAL" },
-      { CKM_DES3_CBC_PAD,	"DES3-CBC-PAD" },
-      { CKM_CDMF_KEY_GEN,	"CDMF-KEY-GEN" },
-      { CKM_CDMF_ECB,		"CDMF-ECB" },
-      { CKM_CDMF_CBC,		"CDMF-CBC" },
-      { CKM_CDMF_MAC,		"CDMF-MAC" },
-      { CKM_CDMF_MAC_GENERAL,	"CDMF-MAC-GENERAL" },
-      { CKM_CDMF_CBC_PAD,	"CDMF-CBC-PAD" },
-      { CKM_MD2,		"MD2" },
-      { CKM_MD2_HMAC,		"MD2-HMAC" },
-      { CKM_MD2_HMAC_GENERAL,	"MD2-HMAC-GENERAL" },
-      { CKM_MD5,		"MD5" },
-      { CKM_MD5_HMAC,		"MD5-HMAC" },
-      { CKM_MD5_HMAC_GENERAL,	"MD5-HMAC-GENERAL" },
-      { CKM_SHA_1,		"SHA-1" },
-      { CKM_SHA_1_HMAC,		"SHA-1-HMAC" },
-      { CKM_SHA_1_HMAC_GENERAL,	"SHA-1-HMAC-GENERAL" },
-      { CKM_RIPEMD128,		"RIPEMD128" },
-      { CKM_RIPEMD128_HMAC,	"RIPEMD128-HMAC" },
-      { CKM_RIPEMD128_HMAC_GENERAL,"RIPEMD128-HMAC-GENERAL" },
-      { CKM_RIPEMD160,		"RIPEMD160" },
-      { CKM_RIPEMD160_HMAC,	"RIPEMD160-HMAC" },
-      { CKM_RIPEMD160_HMAC_GENERAL,"RIPEMD160-HMAC-GENERAL" },
-      { CKM_CAST_KEY_GEN,	"CAST-KEY-GEN" },
-      { CKM_CAST_ECB,		"CAST-ECB" },
-      { CKM_CAST_CBC,		"CAST-CBC" },
-      { CKM_CAST_MAC,		"CAST-MAC" },
-      { CKM_CAST_MAC_GENERAL,	"CAST-MAC-GENERAL" },
-      { CKM_CAST_CBC_PAD,	"CAST-CBC-PAD" },
-      { CKM_CAST3_KEY_GEN,	"CAST3-KEY-GEN" },
-      { CKM_CAST3_ECB,		"CAST3-ECB" },
-      { CKM_CAST3_CBC,		"CAST3-CBC" },
-      { CKM_CAST3_MAC,		"CAST3-MAC" },
-      { CKM_CAST3_MAC_GENERAL,	"CAST3-MAC-GENERAL" },
-      { CKM_CAST3_CBC_PAD,	"CAST3-CBC-PAD" },
-      { CKM_CAST5_KEY_GEN,	"CAST5-KEY-GEN" },
-      { CKM_CAST5_ECB,		"CAST5-ECB" },
-      { CKM_CAST5_CBC,		"CAST5-CBC" },
-      { CKM_CAST5_MAC,		"CAST5-MAC" },
-      { CKM_CAST5_MAC_GENERAL,	"CAST5-MAC-GENERAL" },
-      { CKM_CAST5_CBC_PAD,	"CAST5-CBC-PAD" },
-      { CKM_RC5_KEY_GEN,	"RC5-KEY-GEN" },
-      { CKM_RC5_ECB,		"RC5-ECB" },
-      { CKM_RC5_CBC,		"RC5-CBC" },
-      { CKM_RC5_MAC,		"RC5-MAC" },
-      { CKM_RC5_MAC_GENERAL,	"RC5-MAC-GENERAL" },
-      { CKM_RC5_CBC_PAD,	"RC5-CBC-PAD" },
-      { CKM_IDEA_KEY_GEN,	"IDEA-KEY-GEN" },
-      { CKM_IDEA_ECB,		"IDEA-ECB" },
-      { CKM_IDEA_CBC,		"IDEA-CBC" },
-      { CKM_IDEA_MAC,		"IDEA-MAC" },
-      { CKM_IDEA_MAC_GENERAL,	"IDEA-MAC-GENERAL" },
-      { CKM_IDEA_CBC_PAD,	"IDEA-CBC-PAD" },
-      { CKM_GENERIC_SECRET_KEY_GEN,"GENERIC-SECRET-KEY-GEN" },
-      { CKM_CONCATENATE_BASE_AND_KEY,"CONCATENATE-BASE-AND-KEY" },
-      { CKM_CONCATENATE_BASE_AND_DATA,"CONCATENATE-BASE-AND-DATA" },
-      { CKM_CONCATENATE_DATA_AND_BASE,"CONCATENATE-DATA-AND-BASE" },
-      { CKM_XOR_BASE_AND_DATA,	"XOR-BASE-AND-DATA" },
-      { CKM_EXTRACT_KEY_FROM_KEY,"EXTRACT-KEY-FROM-KEY" },
-      { CKM_SSL3_PRE_MASTER_KEY_GEN,"SSL3-PRE-MASTER-KEY-GEN" },
-      { CKM_SSL3_MASTER_KEY_DERIVE,"SSL3-MASTER-KEY-DERIVE" },
-      { CKM_SSL3_KEY_AND_MAC_DERIVE,"SSL3-KEY-AND-MAC-DERIVE" },
-      { CKM_SSL3_MASTER_KEY_DERIVE_DH,"SSL3-MASTER-KEY-DERIVE-DH" },
-      { CKM_TLS_PRE_MASTER_KEY_GEN,"TLS-PRE-MASTER-KEY-GEN" },
-      { CKM_TLS_MASTER_KEY_DERIVE,"TLS-MASTER-KEY-DERIVE" },
-      { CKM_TLS_KEY_AND_MAC_DERIVE,"TLS-KEY-AND-MAC-DERIVE" },
-      { CKM_TLS_MASTER_KEY_DERIVE_DH,"TLS-MASTER-KEY-DERIVE-DH" },
-      { CKM_SSL3_MD5_MAC,	"SSL3-MD5-MAC" },
-      { CKM_SSL3_SHA1_MAC,	"SSL3-SHA1-MAC" },
-      { CKM_MD5_KEY_DERIVATION,	"MD5-KEY-DERIVATION" },
-      { CKM_MD2_KEY_DERIVATION,	"MD2-KEY-DERIVATION" },
-      { CKM_SHA1_KEY_DERIVATION,"SHA1-KEY-DERIVATION" },
-      { CKM_PBE_MD2_DES_CBC,	"PBE-MD2-DES-CBC" },
-      { CKM_PBE_MD5_DES_CBC,	"PBE-MD5-DES-CBC" },
-      { CKM_PBE_MD5_CAST_CBC,	"PBE-MD5-CAST-CBC" },
-      { CKM_PBE_MD5_CAST3_CBC,	"PBE-MD5-CAST3-CBC" },
-      { CKM_PBE_MD5_CAST5_CBC,	"PBE-MD5-CAST5-CBC" },
-      { CKM_PBE_SHA1_CAST5_CBC,	"PBE-SHA1-CAST5-CBC" },
-      { CKM_PBE_SHA1_RC4_128,	"PBE-SHA1-RC4-128" },
-      { CKM_PBE_SHA1_RC4_40,	"PBE-SHA1-RC4-40" },
-      { CKM_PBE_SHA1_DES3_EDE_CBC,"PBE-SHA1-DES3-EDE-CBC" },
-      { CKM_PBE_SHA1_DES2_EDE_CBC,"PBE-SHA1-DES2-EDE-CBC" },
-      { CKM_PBE_SHA1_RC2_128_CBC,"PBE-SHA1-RC2-128-CBC" },
-      { CKM_PBE_SHA1_RC2_40_CBC,"PBE-SHA1-RC2-40-CBC" },
-      { CKM_PKCS5_PBKD2,	"PKCS5-PBKD2" },
-      { CKM_PBA_SHA1_WITH_SHA1_HMAC,"PBA-SHA1-WITH-SHA1-HMAC" },
-      { CKM_KEY_WRAP_LYNKS,	"KEY-WRAP-LYNKS" },
-      { CKM_KEY_WRAP_SET_OAEP,	"KEY-WRAP-SET-OAEP" },
-      { CKM_SKIPJACK_KEY_GEN,	"SKIPJACK-KEY-GEN" },
-      { CKM_SKIPJACK_ECB64,	"SKIPJACK-ECB64" },
-      { CKM_SKIPJACK_CBC64,	"SKIPJACK-CBC64" },
-      { CKM_SKIPJACK_OFB64,	"SKIPJACK-OFB64" },
-      { CKM_SKIPJACK_CFB64,	"SKIPJACK-CFB64" },
-      { CKM_SKIPJACK_CFB32,	"SKIPJACK-CFB32" },
-      { CKM_SKIPJACK_CFB16,	"SKIPJACK-CFB16" },
-      { CKM_SKIPJACK_CFB8,	"SKIPJACK-CFB8" },
-      { CKM_SKIPJACK_WRAP,	"SKIPJACK-WRAP" },
-      { CKM_SKIPJACK_PRIVATE_WRAP,"SKIPJACK-PRIVATE-WRAP" },
-      { CKM_SKIPJACK_RELAYX,	"SKIPJACK-RELAYX" },
-      { CKM_KEA_KEY_PAIR_GEN,	"KEA-KEY-PAIR-GEN" },
-      { CKM_KEA_KEY_DERIVE,	"KEA-KEY-DERIVE" },
-      { CKM_FORTEZZA_TIMESTAMP,	"FORTEZZA-TIMESTAMP" },
-      { CKM_BATON_KEY_GEN,	"BATON-KEY-GEN" },
-      { CKM_BATON_ECB128,	"BATON-ECB128" },
-      { CKM_BATON_ECB96,	"BATON-ECB96" },
-      { CKM_BATON_CBC128,	"BATON-CBC128" },
-      { CKM_BATON_COUNTER,	"BATON-COUNTER" },
-      { CKM_BATON_SHUFFLE,	"BATON-SHUFFLE" },
-      { CKM_BATON_WRAP,		"BATON-WRAP" },
-      { CKM_ECDSA_KEY_PAIR_GEN,	"ECDSA-KEY-PAIR-GEN" },
-      { CKM_ECDSA,		"ECDSA" },
-      { CKM_ECDSA_SHA1,		"ECDSA-SHA1" },
-      { CKM_ECDH1_DERIVE,	"ECDH1-DERIVE" },
-      { CKM_ECDH1_COFACTOR_DERIVE,"ECDH1-COFACTOR-DERIVE" },
-      { CKM_ECMQV_DERIVE,	"ECMQV-DERIVE" },
-      { CKM_JUNIPER_KEY_GEN,	"JUNIPER-KEY-GEN" },
-      { CKM_JUNIPER_ECB128,	"JUNIPER-ECB128" },
-      { CKM_JUNIPER_CBC128,	"JUNIPER-CBC128" },
-      { CKM_JUNIPER_COUNTER,	"JUNIPER-COUNTER" },
-      { CKM_JUNIPER_SHUFFLE,	"JUNIPER-SHUFFLE" },
-      { CKM_JUNIPER_WRAP,	"JUNIPER-WRAP" },
-      { CKM_FASTHASH,		"FASTHASH" },
-      { CKM_AES_KEY_GEN,	"AES-KEY-GEN" },
-      { CKM_AES_ECB,		"AES-ECB" },
-      { CKM_AES_CBC,		"AES-CBC" },
-      { CKM_AES_MAC,		"AES-MAC" },
-      { CKM_AES_MAC_GENERAL,	"AES-MAC-GENERAL" },
-      { CKM_AES_CBC_PAD,	"AES-CBC-PAD" },
-      { CKM_DSA_PARAMETER_GEN,	"DSA-PARAMETER-GEN" },
-      { CKM_DH_PKCS_PARAMETER_GEN,"DH-PKCS-PARAMETER-GEN" },
-      { CKM_X9_42_DH_PARAMETER_GEN,"X9-42-DH-PARAMETER-GEN" },
-      { NO_MECHANISM, NULL }
+      { CKM_RSA_PKCS_OAEP,	"RSA-PKCS-OAEP",	NULL },
+      { CKM_RSA_X9_31_KEY_PAIR_GEN,"RSA-X9-31-KEY-PAIR-GEN", NULL },
+      { CKM_RSA_X9_31,		"RSA-X9-31",	NULL },
+      { CKM_SHA1_RSA_X9_31,	"SHA1-RSA-X9-31",	NULL },
+      { CKM_RSA_PKCS_PSS,	"RSA-PKCS-PSS",	NULL },
+      { CKM_SHA1_RSA_PKCS_PSS,	"SHA1-RSA-PKCS-PSS",	NULL },
+      { CKM_DSA_KEY_PAIR_GEN,	"DSA-KEY-PAIR-GEN",	NULL },
+      { CKM_DSA,		"DSA",	NULL },
+      { CKM_DSA_SHA1,		"DSA-SHA1", NULL },
+      { CKM_DH_PKCS_KEY_PAIR_GEN,"DH-PKCS-KEY-PAIR-GEN", NULL },
+      { CKM_DH_PKCS_DERIVE,	"DH-PKCS-DERIVE", NULL },
+      { CKM_X9_42_DH_KEY_PAIR_GEN,"X9-42-DH-KEY-PAIR-GEN", NULL },
+      { CKM_X9_42_DH_DERIVE,	"X9-42-DH-DERIVE", NULL },
+      { CKM_X9_42_DH_HYBRID_DERIVE,"X9-42-DH-HYBRID-DERIVE", NULL },
+      { CKM_X9_42_MQV_DERIVE,	"X9-42-MQV-DERIVE", NULL },
+      { CKM_RC2_KEY_GEN,	"RC2-KEY-GEN", NULL },
+      { CKM_RC2_ECB,		"RC2-ECB", NULL },
+      { CKM_RC2_CBC,		"RC2-CBC", NULL },
+      { CKM_RC2_MAC,		"RC2-MAC", NULL },
+      { CKM_RC2_MAC_GENERAL,	"RC2-MAC-GENERAL", NULL },
+      { CKM_RC2_CBC_PAD,	"RC2-CBC-PAD", NULL },
+      { CKM_RC4_KEY_GEN,	"RC4-KEY-GEN", NULL },
+      { CKM_RC4,		"RC4", NULL },
+      { CKM_DES_KEY_GEN,	"DES-KEY-GEN", NULL },
+      { CKM_DES_ECB,		"DES-ECB", NULL },
+      { CKM_DES_CBC,		"DES-CBC", NULL },
+      { CKM_DES_MAC,		"DES-MAC", NULL },
+      { CKM_DES_MAC_GENERAL,	"DES-MAC-GENERAL", NULL },
+      { CKM_DES_CBC_PAD,	"DES-CBC-PAD", NULL },
+      { CKM_DES2_KEY_GEN,	"DES2-KEY-GEN", NULL },
+      { CKM_DES3_KEY_GEN,	"DES3-KEY-GEN", NULL },
+      { CKM_DES3_ECB,		"DES3-ECB", NULL },
+      { CKM_DES3_CBC,		"DES3-CBC", NULL },
+      { CKM_DES3_MAC,		"DES3-MAC", NULL },
+      { CKM_DES3_MAC_GENERAL,	"DES3-MAC-GENERAL", NULL },
+      { CKM_DES3_CBC_PAD,	"DES3-CBC-PAD", NULL },
+      { CKM_CDMF_KEY_GEN,	"CDMF-KEY-GEN", NULL },
+      { CKM_CDMF_ECB,		"CDMF-ECB", NULL },
+      { CKM_CDMF_CBC,		"CDMF-CBC", NULL },
+      { CKM_CDMF_MAC,		"CDMF-MAC", NULL },
+      { CKM_CDMF_MAC_GENERAL,	"CDMF-MAC-GENERAL", NULL },
+      { CKM_CDMF_CBC_PAD,	"CDMF-CBC-PAD", NULL },
+      { CKM_MD2,		"MD2", NULL },
+      { CKM_MD2_HMAC,		"MD2-HMAC", NULL },
+      { CKM_MD2_HMAC_GENERAL,	"MD2-HMAC-GENERAL", NULL },
+      { CKM_MD5,		"MD5", NULL },
+      { CKM_MD5_HMAC,		"MD5-HMAC", NULL },
+      { CKM_MD5_HMAC_GENERAL,	"MD5-HMAC-GENERAL", NULL },
+      { CKM_SHA_1,		"SHA-1", NULL },
+      { CKM_SHA_1_HMAC,		"SHA-1-HMAC", NULL },
+      { CKM_SHA_1_HMAC_GENERAL,	"SHA-1-HMAC-GENERAL", NULL },
+      { CKM_RIPEMD128,		"RIPEMD128", NULL },
+      { CKM_RIPEMD128_HMAC,	"RIPEMD128-HMAC", NULL },
+      { CKM_RIPEMD128_HMAC_GENERAL,"RIPEMD128-HMAC-GENERAL", NULL },
+      { CKM_RIPEMD160,		"RIPEMD160", NULL },
+      { CKM_RIPEMD160_HMAC,	"RIPEMD160-HMAC", NULL },
+      { CKM_RIPEMD160_HMAC_GENERAL,"RIPEMD160-HMAC-GENERAL", NULL },
+      { CKM_CAST_KEY_GEN,	"CAST-KEY-GEN", NULL },
+      { CKM_CAST_ECB,		"CAST-ECB", NULL },
+      { CKM_CAST_CBC,		"CAST-CBC", NULL },
+      { CKM_CAST_MAC,		"CAST-MAC", NULL },
+      { CKM_CAST_MAC_GENERAL,	"CAST-MAC-GENERAL", NULL },
+      { CKM_CAST_CBC_PAD,	"CAST-CBC-PAD", NULL },
+      { CKM_CAST3_KEY_GEN,	"CAST3-KEY-GEN", NULL },
+      { CKM_CAST3_ECB,		"CAST3-ECB", NULL },
+      { CKM_CAST3_CBC,		"CAST3-CBC", NULL },
+      { CKM_CAST3_MAC,		"CAST3-MAC", NULL },
+      { CKM_CAST3_MAC_GENERAL,	"CAST3-MAC-GENERAL", NULL },
+      { CKM_CAST3_CBC_PAD,	"CAST3-CBC-PAD", NULL },
+      { CKM_CAST5_KEY_GEN,	"CAST5-KEY-GEN", NULL },
+      { CKM_CAST5_ECB,		"CAST5-ECB", NULL },
+      { CKM_CAST5_CBC,		"CAST5-CBC", NULL },
+      { CKM_CAST5_MAC,		"CAST5-MAC", NULL },
+      { CKM_CAST5_MAC_GENERAL,	"CAST5-MAC-GENERAL", NULL },
+      { CKM_CAST5_CBC_PAD,	"CAST5-CBC-PAD", NULL },
+      { CKM_RC5_KEY_GEN,	"RC5-KEY-GEN", NULL },
+      { CKM_RC5_ECB,		"RC5-ECB", NULL },
+      { CKM_RC5_CBC,		"RC5-CBC", NULL },
+      { CKM_RC5_MAC,		"RC5-MAC", NULL },
+      { CKM_RC5_MAC_GENERAL,	"RC5-MAC-GENERAL", NULL },
+      { CKM_RC5_CBC_PAD,	"RC5-CBC-PAD", NULL },
+      { CKM_IDEA_KEY_GEN,	"IDEA-KEY-GEN", NULL },
+      { CKM_IDEA_ECB,		"IDEA-ECB", NULL },
+      { CKM_IDEA_CBC,		"IDEA-CBC", NULL },
+      { CKM_IDEA_MAC,		"IDEA-MAC", NULL },
+      { CKM_IDEA_MAC_GENERAL,	"IDEA-MAC-GENERAL", NULL },
+      { CKM_IDEA_CBC_PAD,	"IDEA-CBC-PAD", NULL },
+      { CKM_GENERIC_SECRET_KEY_GEN,"GENERIC-SECRET-KEY-GEN", NULL },
+      { CKM_CONCATENATE_BASE_AND_KEY,"CONCATENATE-BASE-AND-KEY", NULL },
+      { CKM_CONCATENATE_BASE_AND_DATA,"CONCATENATE-BASE-AND-DATA", NULL },
+      { CKM_CONCATENATE_DATA_AND_BASE,"CONCATENATE-DATA-AND-BASE", NULL },
+      { CKM_XOR_BASE_AND_DATA,	"XOR-BASE-AND-DATA", NULL },
+      { CKM_EXTRACT_KEY_FROM_KEY,"EXTRACT-KEY-FROM-KEY", NULL },
+      { CKM_SSL3_PRE_MASTER_KEY_GEN,"SSL3-PRE-MASTER-KEY-GEN", NULL },
+      { CKM_SSL3_MASTER_KEY_DERIVE,"SSL3-MASTER-KEY-DERIVE", NULL },
+      { CKM_SSL3_KEY_AND_MAC_DERIVE,"SSL3-KEY-AND-MAC-DERIVE", NULL },
+      { CKM_SSL3_MASTER_KEY_DERIVE_DH,"SSL3-MASTER-KEY-DERIVE-DH", NULL },
+      { CKM_TLS_PRE_MASTER_KEY_GEN,"TLS-PRE-MASTER-KEY-GEN", NULL },
+      { CKM_TLS_MASTER_KEY_DERIVE,"TLS-MASTER-KEY-DERIVE", NULL },
+      { CKM_TLS_KEY_AND_MAC_DERIVE,"TLS-KEY-AND-MAC-DERIVE", NULL },
+      { CKM_TLS_MASTER_KEY_DERIVE_DH,"TLS-MASTER-KEY-DERIVE-DH", NULL },
+      { CKM_SSL3_MD5_MAC,	"SSL3-MD5-MAC", NULL },
+      { CKM_SSL3_SHA1_MAC,	"SSL3-SHA1-MAC", NULL },
+      { CKM_MD5_KEY_DERIVATION,	"MD5-KEY-DERIVATION", NULL },
+      { CKM_MD2_KEY_DERIVATION,	"MD2-KEY-DERIVATION", NULL },
+      { CKM_SHA1_KEY_DERIVATION,"SHA1-KEY-DERIVATION", NULL },
+      { CKM_PBE_MD2_DES_CBC,	"PBE-MD2-DES-CBC", NULL },
+      { CKM_PBE_MD5_DES_CBC,	"PBE-MD5-DES-CBC", NULL },
+      { CKM_PBE_MD5_CAST_CBC,	"PBE-MD5-CAST-CBC", NULL },
+      { CKM_PBE_MD5_CAST3_CBC,	"PBE-MD5-CAST3-CBC", NULL },
+      { CKM_PBE_MD5_CAST5_CBC,	"PBE-MD5-CAST5-CBC", NULL },
+      { CKM_PBE_SHA1_CAST5_CBC,	"PBE-SHA1-CAST5-CBC", NULL },
+      { CKM_PBE_SHA1_RC4_128,	"PBE-SHA1-RC4-128", NULL },
+      { CKM_PBE_SHA1_RC4_40,	"PBE-SHA1-RC4-40", NULL },
+      { CKM_PBE_SHA1_DES3_EDE_CBC,"PBE-SHA1-DES3-EDE-CBC", NULL },
+      { CKM_PBE_SHA1_DES2_EDE_CBC,"PBE-SHA1-DES2-EDE-CBC", NULL },
+      { CKM_PBE_SHA1_RC2_128_CBC,"PBE-SHA1-RC2-128-CBC", NULL },
+      { CKM_PBE_SHA1_RC2_40_CBC,"PBE-SHA1-RC2-40-CBC", NULL },
+      { CKM_PKCS5_PBKD2,	"PKCS5-PBKD2", NULL },
+      { CKM_PBA_SHA1_WITH_SHA1_HMAC,"PBA-SHA1-WITH-SHA1-HMAC", NULL },
+      { CKM_KEY_WRAP_LYNKS,	"KEY-WRAP-LYNKS", NULL },
+      { CKM_KEY_WRAP_SET_OAEP,	"KEY-WRAP-SET-OAEP", NULL },
+      { CKM_SKIPJACK_KEY_GEN,	"SKIPJACK-KEY-GEN", NULL },
+      { CKM_SKIPJACK_ECB64,	"SKIPJACK-ECB64", NULL },
+      { CKM_SKIPJACK_CBC64,	"SKIPJACK-CBC64", NULL },
+      { CKM_SKIPJACK_OFB64,	"SKIPJACK-OFB64", NULL },
+      { CKM_SKIPJACK_CFB64,	"SKIPJACK-CFB64", NULL },
+      { CKM_SKIPJACK_CFB32,	"SKIPJACK-CFB32", NULL },
+      { CKM_SKIPJACK_CFB16,	"SKIPJACK-CFB16", NULL },
+      { CKM_SKIPJACK_CFB8,	"SKIPJACK-CFB8", NULL },
+      { CKM_SKIPJACK_WRAP,	"SKIPJACK-WRAP", NULL },
+      { CKM_SKIPJACK_PRIVATE_WRAP,"SKIPJACK-PRIVATE-WRAP", NULL },
+      { CKM_SKIPJACK_RELAYX,	"SKIPJACK-RELAYX", NULL },
+      { CKM_KEA_KEY_PAIR_GEN,	"KEA-KEY-PAIR-GEN", NULL },
+      { CKM_KEA_KEY_DERIVE,	"KEA-KEY-DERIVE", NULL },
+      { CKM_FORTEZZA_TIMESTAMP,	"FORTEZZA-TIMESTAMP", NULL },
+      { CKM_BATON_KEY_GEN,	"BATON-KEY-GEN", NULL },
+      { CKM_BATON_ECB128,	"BATON-ECB128", NULL },
+      { CKM_BATON_ECB96,	"BATON-ECB96", NULL },
+      { CKM_BATON_CBC128,	"BATON-CBC128", NULL },
+      { CKM_BATON_COUNTER,	"BATON-COUNTER", NULL },
+      { CKM_BATON_SHUFFLE,	"BATON-SHUFFLE", NULL },
+      { CKM_BATON_WRAP,		"BATON-WRAP", NULL },
+      { CKM_ECDSA_KEY_PAIR_GEN,	"ECDSA-KEY-PAIR-GEN", NULL },
+      { CKM_ECDSA,		"ECDSA", NULL },
+      { CKM_ECDSA_SHA1,		"ECDSA-SHA1", NULL },
+      { CKM_ECDH1_DERIVE,	"ECDH1-DERIVE", NULL },
+      { CKM_ECDH1_COFACTOR_DERIVE,"ECDH1-COFACTOR-DERIVE", NULL },
+      { CKM_ECMQV_DERIVE,	"ECMQV-DERIVE", NULL },
+      { CKM_JUNIPER_KEY_GEN,	"JUNIPER-KEY-GEN", NULL },
+      { CKM_JUNIPER_ECB128,	"JUNIPER-ECB128", NULL },
+      { CKM_JUNIPER_CBC128,	"JUNIPER-CBC128", NULL },
+      { CKM_JUNIPER_COUNTER,	"JUNIPER-COUNTER", NULL },
+      { CKM_JUNIPER_SHUFFLE,	"JUNIPER-SHUFFLE", NULL },
+      { CKM_JUNIPER_WRAP,	"JUNIPER-WRAP", NULL },
+      { CKM_FASTHASH,		"FASTHASH", NULL },
+      { CKM_AES_KEY_GEN,	"AES-KEY-GEN", NULL },
+      { CKM_AES_ECB,		"AES-ECB", NULL },
+      { CKM_AES_CBC,		"AES-CBC", NULL },
+      { CKM_AES_MAC,		"AES-MAC", NULL },
+      { CKM_AES_MAC_GENERAL,	"AES-MAC-GENERAL", NULL },
+      { CKM_AES_CBC_PAD,	"AES-CBC-PAD", NULL },
+      { CKM_DSA_PARAMETER_GEN,	"DSA-PARAMETER-GEN", NULL },
+      { CKM_DH_PKCS_PARAMETER_GEN,"DH-PKCS-PARAMETER-GEN", NULL },
+      { CKM_X9_42_DH_PARAMETER_GEN,"X9-42-DH-PARAMETER-GEN", NULL },
+      { NO_MECHANISM, NULL, NULL }
 };
 
 static const char *
@@ -3455,7 +3465,7 @@ p11_mechanism_to_name(CK_MECHANISM_TYPE mech)
 		if (mi->mech == mech)
 			return mi->name;
 	}
-	snprintf(temp, sizeof(temp), "mechtype-%lu", mech);
+	snprintf(temp, sizeof(temp), "mechtype-%lu", (unsigned long) mech);
 	return temp;
 }
 
