@@ -251,8 +251,8 @@ static int sign_ext(struct sc_pkcs15_object *obj,
 				nid = NID_sha1;
 			else {
 				fprintf(stderr,
-					"Invalid input size (%u bytes)\n",
-					len);
+					"Invalid input size (%lu bytes)\n",
+					(unsigned long) len);
 				return SC_ERROR_INVALID_ARGUMENTS;
 			}
 		}
@@ -293,8 +293,8 @@ static int sign(struct sc_pkcs15_object *obj)
 	if (obj->type == SC_PKCS15_TYPE_PRKEY_RSA
 	 && !(opt_crypt_flags & SC_ALGORITHM_RSA_PAD_PKCS1)
 	 && (size_t)c != key->modulus_length/8) {
-		fprintf(stderr, "Input has to be exactly %d bytes, when using no padding.\n",
-			key->modulus_length/8);
+		fprintf(stderr, "Input has to be exactly %lu bytes, when using no padding.\n",
+			(unsigned long) key->modulus_length/8);
 		return 2;
 	}
 	if (!key->native) {
@@ -454,6 +454,7 @@ int main(int argc, char * const argv[])
 	int do_sign = 0;
 	int action_count = 0;
         struct sc_pkcs15_object *key;
+	sc_context_param_t ctx_param;
 		
 	while (1) {
 		c = getopt_long(argc, argv, "sck:r:i:o:Rp:vw", options, &long_optind);
@@ -508,7 +509,12 @@ int main(int argc, char * const argv[])
 	}
 	if (action_count == 0)
 		print_usage_and_die();
-	r = sc_establish_context(&ctx, app_name);
+
+	memset(&ctx_param, 0, sizeof(ctx_param));
+	ctx_param.ver      = 0;
+	ctx_param.app_name = app_name;
+
+	r = sc_context_create(&ctx, &ctx_param);
 	if (r) {
 		fprintf(stderr, "Failed to establish context: %s\n", sc_strerror(r));
 		return 1;
